.\"  cellc.man
.\"  Copyright (C) 1992, 1993, 1994, 1995, 1997, 1999  J Dana Eckart
.\"
.\"  This program is free software; you can redistribute it and/or modify
.\"  it under the terms of the GNU General Public License as published by
.\"  the Free Software Foundation; either version 1, or (at your option)
.\"  any later version.
.\"
.\"  This program is distributed in the hope that it will be useful,
.\"  but WITHOUT ANY WARRANTY; without even the implied warranty of
.\"  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
.\"  GNU General Public License for more details.
.\"
.\"  You should have received a copy of the GNU General Public License
.\"  along with CELLULAR; see the file COPYING.  If not, write to the 
.\"  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
.\"
.TH cellc 1
.SH NAME
cellc \- 
.I "Cellang"
(cellular automata) compiler
.SH SYNOPSIS
.B cellc
[
.B \-o 
.I outputfile
]
[
.B \-C
]
[
.B \-F
.I filter
]
[
.B \-cpus 
.I n
]
.if n .ti +6n
[
.B \-mv 
.I n[u|s]
]
[
.B \-s 
.I (n|n,...,n)
]
[
.B \-trace 
.I n,...,n
]
.if n .ti +6n
[
.B \-f 
.I n
]
[
.B \-float 
]
file
.SH DESCRIPTION
.I cellc
is the Cellang celluar automata compiler.
.I cellc
compiles and builds the executable for the Cellang program contained
within file.  By default, the executable is placed in 
.IR avcam.out .
.SH OPTIONS
.TP
.BI \-cpus " n"
.br
This indicates the number of CPUs that are available for used on a shared 
memory multi-processor.  If several processors are available,
the code can be substantially faster.  The default is DEFAULT_CPUS while
the maximum is MAX_CPUS.
.TP
.B \-C
.br
Normally the indices for indexed arrays are not checked at run-time.  This
option turns on the run-time index boundary checking.  If an array index is
out of bounds, the automata stops executing.  Range checking cannot be used
in combination with microvectors (see
.B \-mv
below)
.TP
.B \-F
.I filter
.br
Allows a filter program to be compiled into the automata.  The
.I filter
file must be an object file.  In addition, the filter must
.B NOT
contain a
.I main
declaration.  The filter file should include a declaration equivalent
to the C declaration ``int do_filter(void)'', which is called at the
end of each time step.  If 
.I do_filter
returns a non-zero value, the
automata ceases execution.  To reduce the chances for undesired interations,
all other declarations in the filter should be the equivalent of C static
declarations.  The filter should use ``int num_dims(void)'' to determine
the number of dimensions being used by the simulation and
``int dim_size(int dim)'' to return the size of a given dimension.  The
dimensions are numbered starting at zero (0) and the indices for each dimension
also start at zero (0).  The filter
can access a specific cell field value, likewise numbered starting with
zero (0), at a given point in the automata by
using ``long int get_field(int field, int dim_0_index, int dim_1_index, ...)''.
The current time step of the computation is returned by
``unsigned long int get_time(void)''
Be sure to give the equivalent of C extern declarations for
.IR num_dims\,
.IR dim_size\,
and
.IR get_field
in the filter code.  If the
.B \-float
option is also given, then the return type of the ``get_field'' function will
be float instead of long int.
.TP
.BI \-f " n"
.br
Indicates which field of the cell values should be used for graphical display.
The default is to use the first field (
.BI -f " 1"
).
This option overrides a similar option of avcam(MAN_EXT), with
the compiled program displaying only the values of the specified field.
Use of this option has no effect on output generated by specifying the
.B \-o
option to avcam(MAN_EXT).
.TP
.B \-float
.br
Specifies that the automata should be compiled to use floating point values
and arithmetic in computations rather than integer values and arithmetic.
When this option is choosen all cell values, both input and output,
are considered to be floating point values, the range of values specified
in a color map are assumed to be floating point values, and constant values
within the source code for the program are assumed to be floating point values.
.TP
.BI \-mv " n[u|s]"
.br
Invokes the use of microvectors, using
.I n
bits per data item.  All of the values (cell fields, variables, and
intermediate results of calculations) must require no more than
.I n
bits.  The compiler does not check to insure that this is the case, so
if you do not understand this option DO NOT USE IT!
If
.I u
is given, then unsigned microvector operations are used, while
.I s
specifies that signed microvector operations be used.  Unsigned
operations are the default.
This option is NOT available in conjunction with the
.B \-trace
or
.B \-float
options and is not available for programs that use agents.
.TP
.BI \-o " outputfile"
.br
Name the output file
.IR outputfile .
.I outputfile
cannot be the same as 
.I sourcefile 
(the compiler will not overwrite the source file).  The default is
.IR avcam.out .
.TP
.BI \-s " (n|n,...,n)"
.br
The size of each successive dimension of the cellular universe is set to
the corresponding values given.  If only one value is specified, then
all dimensions will be of that size.
It usually best to choose a power of 2 for the dimension size, due to the
computational method used for indexing cells in the automata.
The default is DEFAULT_DIM_SIZE.
.TP
.BI \-trace " n,...,n"
.br
This option compiles the automata so that the intermediate values
created by the program in determining a new cell's value will be printed
to standard error.  The
.I n,...,n
gives the location of the cell within the automata to trace.  Trace values
are printed only for those times specified in a report timefile when the
program is executed using avcam.  Note that tracing can substantially
slow the execution.
.br
.ne 5
.SH FILES
 file                 program text file
 avcam.out            default executable file
 LIB_DIR/cellcpp      program which actually generates the avcam object code
 TMP_DIR/BASE_FILE_NAME*           temporary files
.SH "SEE ALSO"
cellview(MAN_EXT), avcam(MAN_EXT)
.br
J Dana Eckart,
.I "A Cellular Automata Simulation System"
(1997)
.br
J Dana Eckart,
.I "Cellang: Language Reference Manual"
(1997)
.SH DIAGNOSTICS
The diagnostics produced by cellc are intended to be
self-explanatory.
.SH BUGS
The current implementation does not check to ensure that the values assigned 
to fields are within the associated ranges or that indices used in an array
reference are within the bounds of the array.
.PP
Signed microvectors assume a twos complement representation for negative values.
.PP
Although microvectors are intended to
speed the execution of programs, that is not always the result.  If programs
make more than a few decisions, either via if statements or conditional
assignments, then slower rather than faster code is more likely to result.
.PP
When the
.B \-float
option is used 
a decimal point is not required in specifying a floating point literal value,
but if one is used digits must appear before and after the decimal point.
