/* multi.inc
   Copyright (C) 1993, 1994, 1997, 1998  J Dana Eckart
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with CELLULAR; see the file COPYING.  If not, write to the 
   Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

extern void start_automata(void);
extern int field, output, view, filter;

/* Initialize the values of the above variables. */
void init_display(void) {
	int i;

	dim_given = 0;

	range_dim_1_index = 0;
	range_dim_2_index = 1;
	range_dim_3_index = 2;

#if DIMS >= 1
	lower[0] = 0;
	upper[0] = DIM_1_SIZE-1;
#endif
#if DIMS >= 2
	lower[1] = 0;
	upper[1] = DIM_2_SIZE-1;
#endif
#if DIMS >= 3
	lower[2] = 0;
	upper[2] = DIM_3_SIZE-1;
#endif
}

unsigned char cpus = CPUS;

int master_id = CPUS-1;

#if SUN
mutex_t display_mutex;

volatile int nwaiting[2];
int next_barrier = 0;
mutex_t spin_mutex;

void init_barrier(void) {
        nwaiting[0] = nwaiting[1] = 0;
        mutex_init(&spin_mutex, USYNC_PROCESS, NULL);
}

void barrier(void) {
        int this_barrier;

        while (mutex_trylock(&spin_mutex) != 0) ;

        this_barrier = next_barrier;
        nwaiting[this_barrier]++;

        if (nwaiting[this_barrier] >= CPUS) {
                nwaiting[this_barrier] = 0;
                next_barrier = (next_barrier + 1) % 2;
        }

        mutex_unlock(&spin_mutex);
        while (nwaiting[this_barrier] != 0) ;
}
#endif

/* Do the setup for thread synchronization. */
void init_shared(void) {
        int i;

#if SUN
	if (thr_setconcurrency(CPUS)) perror("thr_setconcurrency");
#elif SGI
	if (m_set_procs(CPUS)) perror("m_set_procs");
#endif

#if SUN
	init_barrier();

	/* Setup the display semaphore. */
	if (mutex_init(&display_mutex, USYNC_THREAD, NULL)) 
                perror("mutex_init of display_mutex");
#endif
}

/* Used by create_threads to pick between master and slave. */
#if SUN
void *master_or_slave(void* id) {
#elif SGI
void master_or_slave(void) {
	int id = m_get_myid();
#endif

        if ((int) id == cpus-1) start_automata();
        else cellang_main(output, view, filter, field-1, (int) id);
}

/* Create multiple threads of control. */
void create_threads(void) {
#if SUN
	int i;
#endif

	init_shared();

#if SUN
	master_id = (int) thr_self();
	for (i = 0; i < CPUS-1; i++)
		if (thr_create(NULL, 0, master_or_slave, (void*) i, 
				THR_BOUND, NULL))
			perror("thr_create");
	master_or_slave((void*) (cpus-1));
#elif SGI
	if (m_fork(master_or_slave)) perror("m_fork");
#endif
}
