.\"  avcam.man
.\"  Copyright (C) 1992, 1993, 1994, 1995, 1997, 1998, 1999  J Dana Eckart
.\"
.\"  This program is free software; you can redistribute it and/or modify
.\"  it under the terms of the GNU General Public License as published by
.\"  the Free Software Foundation; either version 1, or (at your option)
.\"  any later version.
.\"
.\"  This program is distributed in the hope that it will be useful,
.\"  but WITHOUT ANY WARRANTY; without even the implied warranty of
.\"  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
.\"  GNU General Public License for more details.
.\"
.\"  You should have received a copy of the GNU General Public License
.\"  along with CELLULAR; see the file COPYING.  If not, write to the 
.\"  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
.\"
.TH avcam 1
.SH NAME
avcam \- abstract virtual cellular automata machine
.SH SYNOPSIS
.B avcam
[
.BI \-r " timefile"
] 
[
.BI \-c " codefile"
]
[
.BI \-seed " n"
]
.if n .ti +6n
[
.B \-o 
] 
[
.B \-F 
] 
[
.BI \-title " string" 
]
.if n .ti +6n
[
(
.B \-iris
| 
((
.B \-x11 
|
.B \-ogl
)
.if n .ti +25n
[
.BI \-display " connection"
] ) ) ]
.if n .ti +6n
[
.BI \-p " n"
] 
[
.BI \-s " (n|n,n|n,n,n)"
] 
[
.BI \-map " mapfile"
]
.if n .ti +6n
[
.BI \-f " n"
]
[
.BI \-dim " [n|n..n]{,n|,n..n}"
]
.SH DESCRIPTION
.I avcam
is the driver which controls the avcam architecture for running cellular
automata programs.  It can either display the results of the run in the
same fashion as
.I cellview
or it can place the cell values on the standard output (
.B \-o
).  In either case, the input to the automata is taken from standard input.
.SH OPTIONS
.TP
.BI \-c " codefile"
Specifies the program to run, where
.B codefile
contains the avcam object code for some Cellang program.  The default
file to use is 
.IR avcam.out .
.TP
.BI \-dim " (n|n..n){,n|,n..n}"
Indicates which of the dimensions are to be used for viewing as well as
which parts of that dimension to display.  The dimensions must be given
in the order they appear in the input.  A single number indicates a
projection on that dimension, while ranges of the form
.I n..n
indicate which cells of the specified dimension to display.  The range must be
increasing.
No spaces should appear anywhere within the dimension indicator.
The number of ranges given for display can be zero (0), one (1) or
two (2); but the number of dimensions specified must be the same as those
given in the input.  The default is to display upto the first two (2)
dimensions in their entirety, as determined by the size specified when
the automata was compiled using cellc(MAN_EXT).
.TP
.BI \-display " connection"
Connect to the X server display,
.IR connection.
.TP
.BI \-f " n"
.br
Indicates which field of the cell values should be used for display.  The
default is to use the first field (
.BI \-f " 1"
).  This option has no effect if the automata was compiled using event
averaging.
.TP
.BI \-map " file"
Uses the map contained in 
.I file 
when displaying cell values.  Entries in the 
.I file 
appear one per line, with the first entry being either an
integer value or an increasing range of integer values (e.g. -12..100).
The second entry is a color indication consisting of either a
triplet of unsigned (positive) integers (0..65535) associated with 
the amount of red, green and blue "pigment" to use to represent the color 
or it is a 0 or 1 (representing white and black respectively) when using
monochrome displays.
Comments about a particular value mapping may appear on the
remainder of the line.
.TP
.B \-o
.br
Place the cell values on the standard output rather than viewing them.  The
default is to view them in the manner of
.IR cellview "."
.TP
.B \-F 
.br
Perform the filtering actions using the filter that was compiled into the
automata.  If the automata was compiled without a filter, this option has
no effect.
.TP
.BI \-p " n"
.br
If 
.I n 
is positive, then this will cause an 
.I n 
second delay between displaying
the cells of the universe of each time step.  Negative values of 
.I n 
are ignored.  The default pause time is 0.
.TP
.BI \-r " timefile"
Indicates when cell values should be reported/sampled.  A 
.B timefile 
is a list of time indications given one per line.  A number indicates a
time when the values should be
reported.  If a ``+'' appears before the number, this causes values to be 
produced at the indicated time intervals.  If a ``!'' appearing after
the number, this indicates the time at which the last set of values should 
be produced and the simulation stops.
Note that a 
.B timefile 
need not contain a ``!'' entry.
.TP
.BI \-s " (n|n,n|n,n,n)"
For the first option, if
.I n
is positive, it is taken as the size of each cell (i.e. the number of pixels
used for each side of a displayed cell) when
the results are first displayed; provided that using this value will
allow all of the cells to be displayed.  The second option allows both
the width and height of displayed cells to be specified for two dimensions while
the third option allows the depth to also be specified in the case of
three dimensions.  The default for this option is
.BI \-s "1"
.TP
.BI \-seed " n"
This sets the initial seed value for the pseudo-random number generator
that yeilds the successive value of the
.I Cellang
read-only pseudo-variable,
.IR random ","
to
.IR n "."
The default seed value is 10.
.TP
.BI \-title " string"
When either the
.B \-x11
or
.B \-iris
option is chosen, this makes
.I string
the title of the window.
.TP
.B \-x11 \-iris \-ogl
View the output in the X-Windows, IRIS Graphis Library, or OpenGLX
windowing environments respectively (
.B \-ogl
is the default).  A window is opened
and the output shown using colored rectangles.
The following key stroke commands are available:
.bf
.nf

       q or Q to quit
       s or S to stop (temporarily)
       c or C to continue (from a stop)
       e or E to toggle the cell field examination window
       n or N to toggle neighborhood window display

.fi
Within the focus of the neighborhood window, typing a single digit will
change the size of the displayed neighborhood.  The values shown in the
neighborhood window are those centered about the chosen cursor location
inside the main viewing window.
Values are printed in the color corresponding
to that value, with the background colored in the value corresponding to
zero (0) (and thus zero values are not displayed).
The values shown in the cell field examination
window are all of the cell field values of the chosen cursor location.
The cell field examination window will NOT come up if fewer than two cell 
fields were declared.  Cursor locations are chosen by clicking
any of the mouse points when the cursor is positioned at the desired point.
The initial cursor location is taken to be the center of the viewing area.
Note that whenever a value is too large to be shown, hyphens
are displayed instead of the value.
.PP
Three (3) dimensions can be viewed in an orthographic projection using the
.B \-ogl
option.  When this is being done, the background is assumed to be black.
It is recommended that the color map associate black with the zero (0) value
so that such cells will NOT be displayed, effectively making them clear.
In addition, the color map entries may contain an additional value (in the
range 0 - 65535) which determines the degree of opacity, higher values
indicating a more opaque color.  Although neither the neighborhood or
cell field examination windows are supported in this mode of display, the
image view may be manipulated using the following key stroke commands:
.bf
.nf

       -> (right arrow) Rotate right about the y axis
       <- (left arrow)  Rotate left about the y axis
        ^ (up arrow)    Rotate right about the x axis
        V (down arrow)  Rotate left about the x axis
        ]               Rotate right about the z axis
        [               Rotate left about the z axis
        Z               Zoom in on the cell view
        z               Zoom out on the cell view

.fi
.br
.ne 5
.SH FILES
 file                            file of color maps
 LIB_DIR/c-map            default color map for x11, iris, and ogl options
 LIB_DIR/bw-map           default black/white map for the x11 option
.SH "SEE ALSO"
cellc(MAN_EXT), cellview(MAN_EXT)
.br
J Dana Eckart,
.I "A Cellular Automata Simulation System"
(1999)
.br
J Dana Eckart,
.I "Cellang: Language Reference Manual"
(1997)
.SH DIAGNOSTICS
The diagnostics produced by cellview are intended to be
self-explanatory.
.SH BUGS
.PP
The maximum size of a color map is 256 entries.  
.PP
Although the viewing window can be resized, the largest allowed cell side
size is 256 pixels.
.PP
When the
.B \-iris
option is used, the cursor must be placed in the window before any of the
output will be displayed.
.PP
The portion of the universe to view is determined by the use of the
.B \-dim
argument.  Ultimately,
the window should be allowed to move over the cells of the
universe dynamically so that different portions of the universe could be
examined more easily.
.PP
When using the
.BR \-x11 ","
.BR \-iris ", or"
.BR \-ogl
options, the two first dimensions are displayed by default.  If a different
view is desired, the
.B \-o
option should be chosen, and its output sent to cellview(MAN_EXT).
.PP
The maximum size of a neighborhood which can be examined using the neighborhood
sub-window is 9, which encompasses a 19 x 19 region centered about the selected
cell.  In addition, only neighbor values between -999 and 999 can be displayed.
.PP
When the
.B \-ogl
option is used, the neighborhood and cell field examination windows
may interact in such a way that some of the data in the windows
may be obscured.
.PP
Neither the neighborhood nor the cell field examination windows are
available when using the
.B \-ogl
option to view a 3D cell universe.
.PP
When using a cell field examination window, be aware that if the cell values
are displayed for only some time steps, that the examination window may
show values for some intermediate time step since the examination window
queries the cellular automaton directly at the time the examination window
was created.  If the examination window remains displayed and the execution
of the automaton is continued, the values shown will come into sync with the
time steps indicated upon the next time step displayed.
.PP
The
.B \-dim
option does not properly limit and yeild the restricted set of cell values
desired when used with the
.B \-o
option on codefiles that were compiled with the
.B \-mv
(microvector) option.
