/* immis.c
   Copyright (C) 1995  Juergen Dollinger, J Dana Eckart

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with CELLULAR; see the file COPYING.  If not, write to the 
   Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Generate the input data used by immis.  A single command line argument
   indicates the size of each of two dimensions of the cell universe.
*/

#include <stdio.h>
#include <stdlib.h>
#include "my_rand.h"

#define USE_WALLS 0

/* Anfangsbedingungen fuer gasimmi.ruhe erzeugen */

int main(int argc, char *argv[]) {
	int n=0, ne=0, se=0, s=0, sw=0, nw=0;
	int which=0, wall=0, color=0;
	int x, y, max_size;

	/* Check usage. */
	if (argc == 1) {
                fprintf(stderr, "%s: size of automaton must be given.\n",
                        argv[0]);
                return 1;
        } else if (argc > 2)
		fprintf(stderr, "%s: extra options ignored.\n", argv[0]);

	max_size = atoi(argv[1]);

	printf("0\n");

	for (x = 0; x < max_size; x++)
	for (y = 0; y < max_size; y++) {

		if (y%2 == 0) which=1;
		else which=0;

		/* Wall or particle? */
		if (0 == x && USE_WALLS)
			wall=1;
		else if( x==1 && y==1 && USE_WALLS )
			wall=1;
		else {
			/* Particle(s)! */

			/* Chance that a particle is placed. */
			int fill;

			/* Type of particle to place. */
			int particle = 1;
			if(my_random()&16) particle = 2;

			wall=0;

			fill = my_random()%100;
			if (fill < 70) n = particle; else n=0 ;
			fill = my_random()%100;
			if (fill < 70) ne = particle; else ne=0 ;
			fill = my_random()%100;
			if (fill < 70) se = particle; else se=0 ;
			fill = my_random()%100;
			if (fill < 70) s = particle; else s=0 ;
			fill = my_random()%100;
			if (fill < 70) sw = particle; else sw=0 ;
			fill = my_random()%100;
			if (fill < 70) nw = particle; else nw=0 ;

		}

		if (wall) color=3;
		else {
			int one=0, two=0;
			if (n==1) one++;
			else if (n==2) two++;
			if (ne==1) one++;
			else if (ne==2) two++;
			if (se==1) one++;
			else if (se==2) two++;
			if (s==1) one++;
			else if (s==2) two++;
			if (sw==1) one++;
			else if (sw==2) two++;
			if (nw==1) one++;
			else if (nw==2) two++;

			if (two>one) color=2;
			else if (one>two) color=1;
			else color=0;
		}

		/* Write cell location and color. */
		printf("[%d, %d] = %d, ", x, y,color);
		printf("%d, %d, %d, %d, %d, %d, %d, %d, ",
			which, wall, n, ne, se, s, sw, nw);
		printf("%ld", (my_random()&16)/16);
		printf("\n");
	}

	return 0;
}
