/* mgas.c
   Copyright (C) 1992, 1993, 1994  J Dana Eckart

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with CELLULAR; see the file COPYING.  If not, write to the 
   Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Generate the input data used by mgas.  A single command line argument
   indicates the size of each of two dimensions of the cell universe.
*/

#include <stdio.h>
#include <stdlib.h>
#include "my_rand.h"

int main(int argc, char *argv[]) {
	int max_size, x, y;

	/* Check usage. */
	if (argc == 1) {
                fprintf(stderr, "%s: size of automaton must be given.\n",
                        argv[0]);
                return 1;
        } else if (argc > 2)
		fprintf(stderr, "%s: extra options ignored.\n", argv[0]);

	max_size = atoi(argv[1]);

	printf("0\n");

	for (x = 0; x < max_size; x++)
		for (y = 0; y < max_size; y++) {
			/* Write cell location. */
			printf("[%d, %d] = ", x, y);

			/* Write "which" field value. */
			if (x%2 == 0 && y%2 == 0)
				printf("3");
			else if (x%2 == 0 && y%2 == 1)
				printf("1");
			else if (x%2 == 1 && y%2 == 0)
				printf("2");
			else
				printf("0");

			/* Write "rand" field value. */
			printf(", %ld", my_random()%2);

			/* Write "gas" field value. */
			if (0.33*max_size < x && x < 0.66*max_size
					      &&
			    0.33*max_size < y && y < 0.66*max_size)
				printf(", 1");

			printf("\n");
		}

	return 0;
}
