.\"  cellview.man
.\"  Copyright (C) 1992, 1993, 1994, 1995, 1997, 1998, 1999  J Dana Eckart
.\"
.\"  This program is free software; you can redistribute it and/or modify
.\"  it under the terms of the GNU General Public License as published by
.\"  the Free Software Foundation; either version 1, or (at your option)
.\"  any later version.
.\"
.\"  This program is distributed in the hope that it will be useful,
.\"  but WITHOUT ANY WARRANTY; without even the implied warranty of
.\"  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
.\"  GNU General Public License for more details.
.\"
.\"  You should have received a copy of the GNU General Public License
.\"  along with CELLULAR; see the file COPYING.  If not, write to the 
.\"  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
.\"
.TH cellview 1
.SH NAME
cellview \- cellular automata viewer
.SH SYNOPSIS
.B cellview
[
.BI \-title " string"
]
.if n .ti +9n
[
.B \-iris
|
.if n .ti +10n
(
(
.B \-ogl
|
.B \-x11
)
[
.BI \-display " connection"
])]
.if n .ti +9n
[
.BI \-p " n"
]
[
.BI \-s " (n|n,n|n,n,n)"
]
[
.BI \-map " file"
]
.if n .ti +9n
[
.BI \-f " n"
]
[
.B \-float
]
.BI \-dim " (n|n..n){,n|,n..n}"
.SH DESCRIPTION
.I cellview
is the viewer for looking at the output of Cellang programs.
.I cellview
uses a map of values to colors 
to display one (1) or two (2) dimensions of the output of a Cellang 
program;
even though the output of the Cellang program may contain far more than 
two (2) dimensions.  The input to
.I cellview
is always taken from standard input.
Upon display, the lower left hand corner of the view corresponds to the
first index in each (if more than one) range of the dimension(s) being 
displayed.
At time zero (0) all cells default to the color mapped from the
zero (0) value, except when specified otherwise by the input.  The current
time is displayed in the lower left hand corner.
.SH OPTIONS
.TP
.BI \-dim " [n|n..n]{,n|,n..n}"
Indicates which of the dimensions are to be used for viewing as well as
which parts of that dimension to display.  The dimensions must be given
in the order they appear in the input.  A single number indicates a
projection on that dimension, while ranges of the form
.I n..n
indicate which cells of the specified dimension to display.  The range must be
increasing.
No spaces should appear anywhere within the dimension indicator.
The number of ranges given for display can be zero (0), one (1) or 
two (2); but the number of dimensions specified must be the same as those
given in the input.
.TP
.BI \-display " connection"
Connect to the X or OpenGL server display,
.IR connection "."
.TP
.BI \-f " n"
.br
Indicates which field of the cell values should be used for display.  The
default is to use the first field (
.BI -f " 1"
).  If the field is not given, then the current value remains unchanged.
.TP
.B \-float
Indicates that the cell field values and the value ranges specified in
the color-map
(see
.B \-map
option)
will be floating point values.  Note that when this option is used, and if
ranges of values are specified in the color-map, then both the upper and lower
bounds of the range MUST contain a decimal point with at least one digit on
each side of the decimal.
.TP
.BI \-map " file"
Uses the map contained in 
.I file 
when displaying cell values.  Entries in the 
.I file 
appear one per line, with the first entry being either an
integer value or an increasing range of integer values (e.g. -12..100).
The second entry is a color indication consisting of either a
triplet of unsigned (positive) 
integers (0..65535) associated with 
the amount of red, green and blue "pigment" to use to represent the color 
or it is a 0 or 1 (representing white and black respectively) when using
monochrome displays.
Comments about a particular value mapping may appear on the
remainder of the line.
.TP
.BI \-p " n"
.br
If 
.I n 
is positive, then this will cause an 
.I n 
second delay between displaying
the cells of the universe of each time step.  Negative values of 
.I n 
are ignored.  The default pause time is 0.
.TP
.BI \-s " (n|n,n|n,n,n)"
For the first option, if
.I n
is positive, it is taken as the size of each cell (i.e. the number of pixels 
used for each side of a displayed cell) when
the results are first displayed; provided that using this value will
allow all of the cells to be displayed.  The second option allows both 
the width and height of displayed cells to be specified for two dimensions while
the third option allows the depth to also be specified in the case of
three dimensions.  The default for this option is 
.BI \-s "1"
.TP
.BI \-title " string"
When either the
.BR \-x11 ","
.B \-ogl
or
.B \-iris
option is chosen, this makes
.I string
the title of the window.
.TP
.B \-x11 \-iris \-ogl
View the output in the X-Windows, IRIS Graphis Library, or OpenGL
windowing environments respectively (
.B \-ogl
is the default if available).  A window is opened
and the output shown using colored rectangles.
The following key stroke commands are available:
.bf
.nf

       q or Q to quit
       s or S to stop (temporarily)
       c or C to continue (from a stop)
       n or N to toggle neighborhood window display

.fi
Within the focus of the neighborhood window, typing a single digit will
change the size of the displayed neighborhood.  The values shown in the
neighborhood window are those centered about the chosen cursor location
inside the main viewing window.
Values are printed in the color corresponding
to that value, with the background colored in the value corresponding to
zero (0) (and thus zero values are not displayed).
Cursor locations are chosen by clicking
any of the mouse points when the cursor is positioned at the desired point.
The initial cursor location is taken to be the center of the viewing area.
Note that whenever a value is too large to be shown, hyphens
are displayed instead of the value.
.PP
Three (3) dimensions can be viewed in an orthographic projection using the
.B \-ogl
option.  When this is being done, the background is assumed to be black.
It is recommended that the color map associate black with the zero (0) value
so that such cells will NOT be displayed, effectively making them clear.
In addition, the color map entries may contain an additional value (in the
range 0 - 65535) which determines the degree of opacity, higher values
indicating a more opaque color.  Although the neighborhood
window is not supported in this mode of display, the
image view may be manipulated using the following key stroke commands:
.bf
.nf

       -> (right arrow) Rotate right about the y axis
       <- (left arrow)  Rotate left about the y axis
        ^ (up arrow)    Rotate right about the x axis
        V (down arrow)  Rotate left about the x axis
        ]               Rotate right about the z axis
        [               Rotate left about the z axis
        Z               Zoom in on the cell view
        z               Zoom out on the cell view

.fi
.br
.ne 5
.SH FILES
 file                            file of color maps
 LIB_DIR/c-map            default color map for x11, iris, and ogl options
 LIB_DIR/bw-map           default black/white map for the x11 option
.SH "SEE ALSO"
cellc(MAN_EXT), avcam(MAN_EXT)
.br
J Dana Eckart,
.I "A Cellular Automata Simulation System"
(1999)
.br
J Dana Eckart,
.I "Cellang: Language Reference Manual"
(1997)
.SH DIAGNOSTICS
The diagnostics produced by cellview are intended to be
self-explanatory.
.SH BUGS
.PP
The maximum size of a color map is 256 entries.  
.PP
Although the viewing window can be resized, the largest allowed cell side
size is 256 pixels.
.PP
When the
.B \-iris
option is used, the cursor must be placed in the window before any of the
output will be displayed.
.PP
The portion of the universe to view is determined by the use of the
.B \-dim
argument.  Ultimately,
the window should be allowed to move over the cells of the
universe dynamically so that different portions of the universe could be 
examined more easily.
.PP
The maximum size of a neighborhood which can be examined using the neighborhood
sub-window is 9, which encompasses a 19 x 19 region centered about the selected
cell.  In addition, only neighbor values between -999 and 999 can be displayed.
.PP
The neighborhood window is not available when using the
.B \-ogl
option to view a 3D cell universe.
