/* x11.c
   Copyright (C) 1992, 1993, 1994, 1997, 1999  J Dana Eckart
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with CELLULAR; see the file COPYING.  If not, write to the 
   Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>

#if X11

#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <X11/Xos.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include "view.h"

/* Font for displaying the time in the time window. */
#define FONT  "10x20"
static int charheight;

/* Variables used by all the windows. */
static XFontStruct 		*font_info;
static Display 			*dpy;
static int			screen;
static GC			gc;
/* DANA ECKART
static GC			color_gc[MAX_MAP_SIZE];
*/

static char			*x11_title;

static Colormap cmap;
static unsigned long int 	pixels[MAX_MAP_SIZE];
static unsigned long int 	Black_Pixel, White_Pixel;

static Pixmap 			cell_pixmap;

static Window			rootWindow;

/* Variables for controlling the main viewing window. */
static Window  			mainWindow;
static XSetWindowAttributes 	xswa;
static int			cell_side_x = MAX_CELL_SIDE;
static int			cell_side_y = MAX_CELL_SIDE;
static float			cell_side_ratio = 1.0;
static int 			time_window_height = 0;
static unsigned int		mainWidth = 0, mainHeight = 0;

/* Variables for controlling the neighborhood sub-window. */
static Window   neighborWindow = (Window) NULL;
static int 	neighborhood = 1;		/* size of the neighborhood. */
static int 	neighbor_x, neighbor_y;		/* x, y neighborhood viewing 
								co-ordinates. */
static unsigned int neighborWidth, neighborHeight;

#if COMBINED

/* Variables for controlling the neighborhood sub-window. */
static Window   examineWindow = (Window) NULL;
static unsigned int examineWidth, examineHeight;

/* Create the cell field examination window. */
static void CreateExamineWindow(void) {
	XSizeHints sizehints;

	examineWidth = charheight * (field_name_length + MAX_VALUE_WIDTH + 5)/2;
	examineHeight = charheight * field_count * 2;

	examineHeight += time_window_height;	/* For the coordinates. */

	/* Configure the window size hints. */
	sizehints.flags = PMinSize | PMaxSize | PSize;
	sizehints.x = 0;
	sizehints.y = 0;
	sizehints.width = examineWidth;
	sizehints.height = examineHeight;
	sizehints.min_width = sizehints.width;
	sizehints.min_height = sizehints.height;
	sizehints.max_width = sizehints.width;
	sizehints.max_height = sizehints.height;

	examineWindow = 
		XCreateWindow(
			dpy, rootWindow,
			0, 0,
			sizehints.width, sizehints.height,
			1,
			DefaultDepth(dpy, screen),
			InputOutput,
			DefaultVisual(dpy, screen),
			CWEventMask | CWBackPixel | CWBorderPixel | CWColormap,
			&xswa);

	if (!examineWindow)
		error("Unable to open a cell field examination window",
		      (char*) NULL);

	/* Define the window's name. */
	XSetStandardProperties(dpy, examineWindow, 
			       x11_title, x11_title,
			       None, NULL, 0, &sizehints);

	XMapWindow(dpy, examineWindow);
}

/* Display the values of the cell fields. */
static void display_cell_field_values(void) {
	int i;
	char coordinate_string[MAX_STRING_SIZE];

	/* Write the cooridnates. */
	XSetForeground(dpy, gc, White_Pixel);
	XFillRectangle(dpy, examineWindow, gc, 
		       0, examineHeight - time_window_height, 
		       examineWidth, time_window_height);

	if (range_dim_given > 1)
		sprintf(coordinate_string, "< %d, %d >",
			neighbor_x + lower[range_dim_1_index],
			neighbor_y + lower[range_dim_2_index]);
	else
		sprintf(coordinate_string, "< %d >",
			neighbor_x + lower[range_dim_1_index]);

	XSetForeground(dpy, gc, Black_Pixel);
	XDrawString(dpy, examineWindow, gc, 
		    time_window_height/2,
		    examineHeight - (int) (0.25 * time_window_height), 
	            coordinate_string, strlen(coordinate_string));

	/* Clear the contents of the window. */
	XSetForeground(dpy, gc, Black_Pixel);
	XFillRectangle(dpy, examineWindow, gc, 
		       0, 0, 
		       examineWidth, examineHeight - time_window_height);

	XSetForeground(dpy, gc, White_Pixel);

	/* Draw the horizontal lines. */
	for (i = field_count; i > 0; i--)
		XDrawLine(dpy, examineWindow, gc,
			  0, i*charheight*2,
			  examineWidth, i*charheight*2);

	/* Display the cell field values. */
	for (i = 0; i < field_count; i++) {
		char string[MAX_STRING_SIZE];
		char field_name[MAX_STRING_SIZE];
		char field_name_value[MAX_STRING_SIZE];

		{
			int x, y, z;

			if (range[0] == 1) x = lower[0];
			else x = neighbor_x;

			if (range[1] == 1) y = lower[1];
			else if (range[0] == 1) y = neighbor_x;
			else y = neighbor_y;

			if (range[2] == 1) z = lower[2];
			else z = neighbor_y;

			if (use_floats)
				sprintf(string, "%f",
					cell_field_float_value(i, x, y, z));
			else
				get_string_for_value(string,
					             cell_field_value(i,
								      x, y, z),
					             false);
		}
		strncpy(field_name, &field_names[i*field_name_length],
			field_name_length);
		field_name[field_name_length] = '\0';
		sprintf(field_name_value, "%s: %s", field_name, string);

		XDrawString(dpy, examineWindow, gc, 
	    		    charheight*2/3, i*2*charheight + charheight*3/2,
	            	    field_name_value, strlen(field_name_value));
	}
	XSync(dpy, False);
}
#endif

/* Create the sub-window dedicated to showing the numeric values of a
   neighborhood.
*/
static void CreateNeighborWindow(void) {
	XSizeHints sizehints;

	neighborWidth = (2*neighborhood+1) * charheight * MAX_FIELD_VALUE_WIDTH;
	neighborHeight = (range_dim_given > 1 ? neighborWidth :
					        charheight*MAX_FIELD_VALUE_WIDTH);

	neighborHeight += time_window_height;	/* For the coordinates. */

	/* Configure the window size hints. */
	sizehints.flags = PSize;
	sizehints.x = 0;
	sizehints.y = 0;
	sizehints.width = neighborWidth;
	sizehints.height = neighborHeight;

	neighborWindow = 
		XCreateWindow(
			dpy, rootWindow,
			0, 0,
			sizehints.width, sizehints.height,
			1,
			DefaultDepth(dpy, screen),
			InputOutput,
			DefaultVisual(dpy, screen),
			CWEventMask | CWBackPixel | CWBorderPixel | CWColormap,
			&xswa);

	if (!neighborWindow)
		error("Unable to open a neighborhood window", (char*) NULL);

	/* Define the window's name. */
	XSetStandardProperties(dpy, neighborWindow, 
			       x11_title, x11_title,
			       None, NULL, 0, &sizehints);

	XMapWindow(dpy, neighborWindow);
}

/* Display the values of the appropriate neighborhood. */
static void display_neighbor_values(void) {
	int i, j;
	char coordinate_string[MAX_STRING_SIZE];

	/* Write the cooridnates. */
	XSetForeground(dpy, gc, White_Pixel);
	XFillRectangle(dpy, neighborWindow, gc, 
		       0, neighborHeight - time_window_height, 
		       neighborWidth, time_window_height);

	if (range_dim_given > 1)
		sprintf(coordinate_string, "< %d, %d >",
			neighbor_x + lower[range_dim_1_index],
			neighbor_y + lower[range_dim_2_index]);
	else
		sprintf(coordinate_string, "< %d >",
			neighbor_x + lower[range_dim_1_index]);

	XSetForeground(dpy, gc, Black_Pixel);
	XDrawString(dpy, neighborWindow, gc, 
		    time_window_height/2,
		    neighborHeight - (int) (0.25 * time_window_height), 
	            coordinate_string, strlen(coordinate_string));

	/* Clear the contents of the window. */
	XSetForeground(dpy, gc, Black_Pixel);
	XFillRectangle(dpy, neighborWindow, gc, 
		       0, 0, 
		       neighborWidth, neighborHeight - time_window_height);

	XSetForeground(dpy, gc, White_Pixel);

	/* Draw the horizontal lines (if need be). */
	if (range_dim_given > 1) for (i = 2*neighborhood; i > 0; i--)
		XDrawLine(dpy, neighborWindow, gc,
			  0, i*charheight*MAX_FIELD_VALUE_WIDTH,
			  neighborWidth, i*charheight*MAX_FIELD_VALUE_WIDTH);

	/* Draw the vertical lines. */
	for (i = 2*neighborhood; i > 0; i--)
		XDrawLine(dpy, neighborWindow, gc,
			  i*charheight*MAX_FIELD_VALUE_WIDTH, 0,
			  i*charheight*MAX_FIELD_VALUE_WIDTH, 
			  neighborHeight - time_window_height);

	/* Display the neighbor values. */
	if (range_dim_given == 1)
		for (i = -neighborhood; i <= neighborhood; i++) {
			int x = (neighbor_x + i + range[range_dim_1_index]) %
						range[range_dim_1_index];
			char string[MAX_STRING_SIZE];

			if (use_floats)
				sprintf(string, "%f", cell_float_vals[x]);
			else
				get_string_for_value(string,
						     cell_vals[x], true);

			XSetForeground(dpy, gc, cell_map_vals[x]);
			XDrawString(dpy, neighborWindow, gc, 
		    		(neighborhood+i)*charheight*MAX_FIELD_VALUE_WIDTH +
					charheight*MAX_FIELD_VALUE_WIDTH/4,
				(MAX_FIELD_VALUE_WIDTH+1)*charheight/2-4,
	            		string, strlen(string));
	}
	else for (i = -neighborhood; i <= neighborhood; i++)
		for (j = -neighborhood; j <= neighborhood; j++) {
			int x = (neighbor_x + i + range[range_dim_1_index]) %
						range[range_dim_1_index];
			int y = (neighbor_y + j + range[range_dim_2_index]) %
						range[range_dim_2_index];
			char string[MAX_STRING_SIZE];
			int index = x * range[range_dim_2_index] + y;

			if (use_floats)
				sprintf(string, "%f", cell_float_vals[index]);
			else
				get_string_for_value(string,
						     cell_vals[index], true);

			XSetForeground(dpy, gc,
				       cell_map_vals[x*range[range_dim_2_index]+y]);
			XDrawString(dpy, neighborWindow, gc, 
		    		(neighborhood+i)*charheight*MAX_FIELD_VALUE_WIDTH +
					charheight*MAX_FIELD_VALUE_WIDTH/4,
		    		(neighborhood-j)*charheight*MAX_FIELD_VALUE_WIDTH +
				(MAX_FIELD_VALUE_WIDTH+1)*charheight/2-4,
	            		string, strlen(string));
		}
	XSync(dpy, False);
}

/* Resize the neighborhood sub-window. */
static void resize_neighborhood(void) {

	neighborWidth = (2*neighborhood+1) * charheight * MAX_FIELD_VALUE_WIDTH;
	neighborHeight = (range_dim_given > 1 ? neighborWidth :
					        charheight*MAX_FIELD_VALUE_WIDTH);

	neighborHeight += time_window_height;	/* For the coordinates. */

	XResizeWindow(dpy, neighborWindow, neighborWidth, neighborHeight);
	XSync(dpy, False);
}

/* Do any cleanup for x11 windows. */
void finish_x11(void) {
	XFreeColormap(dpy, cmap);
	XFlush(dpy);
	XCloseDisplay(dpy);
}

/* Display the current time step at the bottom of the window. */
static void display_time(void) {
	char time_string[MAX_STRING_SIZE];

	/* Clear the time display part of the window. */
	XSetForeground(dpy, gc, White_Pixel);
	XFillRectangle(dpy, mainWindow, gc, 
		       0, cell_side_y*range[range_dim_2_index],
		       mainWidth, time_window_height);

	/* Display the new current time step. */
	XSetForeground(dpy, gc, Black_Pixel);
	sprintf(time_string, "time = %d", _time_step);
	XDrawString(dpy, mainWindow, gc, 
		    time_window_height/2,
		    cell_side_y * range[range_dim_2_index] + 
				(int) (0.75 * time_window_height), 
	            time_string, strlen(time_string));
}

/* Update (display) the color of a single cell. */
void update_x11(long int x, long int y, celltype color_index) {
	long int i = x - lower[range_dim_1_index];
	long int j = y - lower[range_dim_2_index];

#if COMBINED && MAX_CPUS > 1
	if (cpus > 1)
#if SUN
		while (0 != mutex_trylock(&display_mutex));
#elif SGI
		m_lock();
#else
		;
#endif
#endif

/* DANA ECKART
	XFillRectangle(dpy, cell_pixmap, color_gc[(int) color_index],
*/
        XSetForeground(dpy, gc, color_index);
	XFillRectangle(dpy, cell_pixmap, gc,
			i*cell_side_x, 
			(range[range_dim_2_index]-1-j)*cell_side_y, 
			cell_side_x, cell_side_y);

#if COMBINED && MAX_CPUS > 1
	if (cpus > 1)
#if SUN
		mutex_unlock(&display_mutex);
#elif SGI
		m_unlock();
#else
		;
#endif
#endif
}

/* Display all the cell_map_vals (and the neighborhood cell_vals) in the 
   universe.
*/
void display_x11_cells(void) {
	XCopyArea(dpy, cell_pixmap, mainWindow, gc, 0, 0,
		  range[range_dim_1_index]*cell_side_x, 
		  range[range_dim_2_index]*cell_side_y, 0, 0);
	XSync(dpy, False);

	/* May need to update other windows too. */
	if (neighborWindow) display_neighbor_values();
#if COMBINED
	if (examineWindow) display_cell_field_values();
#endif
}

/* Reset the pixel map with cell "colors". */
static void redisplay_cells(void) {
	register long int i, j;

	for (i = 0; i < range[range_dim_1_index]; i++)
		for (j = 0; j < range[range_dim_2_index]; j++)
			update_x11(i, j, 
				cell_map_vals[i*range[range_dim_2_index]+j]);

	display_x11_cells();
}

/* Handle the events for the neighborhood sub-window. */
static void sub_window_event(XEvent *event) {
	XExposeEvent *ee = (XExposeEvent*) event;
        XConfigureEvent *ce = (XConfigureEvent*) event;
	XKeyPressedEvent *ke = (XKeyPressedEvent *) event;
	char keybuff[1];
        int nchar;

	switch(event->type) {
       	case ConfigureNotify:
		/* Don't resize this window.  If an attempt was made to
		   resize the neighborhood window directly, reset it.
		*/
		if (ce->width != neighborWidth || ce->height != neighborHeight) {
			resize_neighborhood();
			display_neighbor_values();
		}
		break;

	case Expose:
		if (ee->count) break;

	case VisibilityNotify:
		display_neighbor_values();
       		break;

	case ButtonPress:
       		break;

	case MappingNotify:
		XRefreshKeyboardMapping((XMappingEvent*) event);
		break;

	case KeyPress:
		nchar = XLookupString(ke, keybuff, 1, NULL, NULL);
               	if (nchar > 0) {
               		switch (keybuff[0]) {
                       	case '0':
                       	case '1':
                       	case '2':
                       	case '3':
                       	case '4':
                       	case '5':
                       	case '6':
                       	case '7':
                       	case '8':
                       	case '9':
				neighborhood = keybuff[0]-'0';
				resize_neighborhood();
				display_neighbor_values();
				break;

			default:
				break;
                       	}
       	       	}
       	       	break;

	default:
		break;
	}
}

/* Handle the events for the main window. */
static void main_window_event(int *stopped, XEvent *event) {
	XExposeEvent *ee = (XExposeEvent*) event;
        XConfigureEvent *ce = (XConfigureEvent*) event;
	XButtonPressedEvent *be = (XButtonPressedEvent *) event;
	XKeyPressedEvent *ke = (XKeyPressedEvent *) event;
	char keybuff[1];
        int nchar;

	switch(event->type) {
       	case ConfigureNotify:
		/* Don't resize the window if both the width and
		   the height are the same.
		*/
		if (ce->width == mainWidth && ce->height == mainHeight)
			break;

		/* The window needs to be resized. */
		ce->height -= time_window_height;
		cell_side_x = (ce->width / range[range_dim_1_index] > 
				ce->height / range[range_dim_2_index] ?
				ce->height / range[range_dim_2_index] : 
				ce->width / range[range_dim_1_index]);

		if (cell_side_ratio < 1) {
			if (cell_side_x > MAX_CELL_SIDE) 
				cell_side_x = MAX_CELL_SIDE;

			/* Make sure cell_vals are "visible". */
			if (cell_side_x <= 0) cell_side_x = 1;

			cell_side_y = cell_side_x * cell_side_ratio;
		}
		else {
			cell_side_y = cell_side_x * cell_side_ratio;

			if (cell_side_y > MAX_CELL_SIDE) 
				cell_side_y = MAX_CELL_SIDE;

			/* Make sure cell_vals are "visible". */
			if (cell_side_y <= 0) cell_side_y = 1;

			cell_side_x = cell_side_y / cell_side_ratio;
		}

		/* Since the size just chosen for the window may
		   not yield an integer number of pixels for the
		   sides of cells, reset the window size to just
		   fit the new cell_side_x and cell_side_y values.
		*/
		mainWidth = range[range_dim_1_index]*cell_side_x;
		mainHeight = range[range_dim_2_index]*cell_side_y + 
			      	time_window_height;

		XResizeWindow(dpy, mainWindow, mainWidth, mainHeight);

		/* Free and create a new pixel map. */
		XFreePixmap(dpy, cell_pixmap);
		cell_pixmap = 
			XCreatePixmap(dpy, mainWindow,
				    range[range_dim_1_index]*cell_side_x,
				    range[range_dim_2_index]*cell_side_y,
				    DefaultDepth(dpy, screen));

		redisplay_cells();
		display_time();
       		break;

	case Expose:
		if (ee->count) break;

	case VisibilityNotify:
		display_x11_cells();
		display_time();
       		break;

	case ButtonPress:
#if COMBINED
		if (neighborWindow || examineWindow) {
#else
		if (neighborWindow) {
#endif
			int old_x = neighbor_x;
			int old_y = neighbor_y;

			neighbor_x = (be->x/cell_side_x + 
					range[range_dim_1_index]) %
						range[range_dim_1_index];

			if (range_dim_given > 1)
				neighbor_y = (-be->y/cell_side_y - 1 + 
						range[range_dim_2_index]) %
						range[range_dim_2_index];

			if (neighbor_x < 0 ||
				neighbor_x >= range[range_dim_1_index] ||
				(range_dim_given > 1 &&
					(neighbor_y < 0 || 
                                	 neighbor_y >= range[range_dim_2_index]))) {
				neighbor_x = old_x;
				neighbor_y = old_y;
			}
		}
#if COMBINED
		if (examineWindow) display_cell_field_values();
#endif
		if (neighborWindow) display_neighbor_values();
		break;

	case MappingNotify:
		XRefreshKeyboardMapping((XMappingEvent*) event);
		break;

	case KeyPress:
		nchar = XLookupString(ke, keybuff, 1, NULL, NULL);
               	if (nchar > 0) {
               		switch (keybuff[0]) {
#if COMBINED
                       	case 'e':
                       	case 'E':
				/* Cell field examination window */
				if (field_count == 1)
					;
				else if (!examineWindow) {
					CreateExamineWindow();
					display_cell_field_values();
				} else {
					XDestroyWindow(dpy, examineWindow);
					examineWindow = (Window) NULL;
				}
				break;
#endif
                       	case 'n':
                       	case 'N':
				/* Neighborhood window */
				if (!neighborWindow) {
					CreateNeighborWindow();
					display_neighbor_values();
				} else {
					XDestroyWindow(dpy, neighborWindow);
					neighborWindow = (Window) NULL;
				}
				break;
               		case 'q':
               		case 'Q':
				/* Quit */
				exit_signaled(0);
               		case 's':
               		case 'S':
				/* Stop */
                       		*stopped = true;
				break;
               		case 'c':
               		case 'C':
				/* Continue */
                       		*stopped = false;
				break;
			default:
				break;
                       	}
       	       	}
       	       	break;

	default:
		break;
	}
}

/* Check for, and perform events as they arise. */
static void do_event(void) {
	XEvent event;
	boolean stopped = false;

	/* If there are events, handled them. */
	while (XPending(dpy) || stopped) {
		XNextEvent(dpy, &event);
		if (event.xany.window == neighborWindow)
			sub_window_event(&event);
#if COMBINED
		else if (event.xany.window == examineWindow)
			/* Assume the event is a VisibilityNotify. */
			display_cell_field_values();
#endif
		else if (event.xany.window == mainWindow)
			main_window_event(&stopped, &event);
	}
}

/* Stores the red, green and blue portions that make up a color. */
celltype setcolor_x11(unsigned int red, unsigned int green, unsigned int blue, int index) {
	XColor color;
	color.pixel = pixels[index];
	color.red = red;
	color.green = green;
	color.blue = blue;
	color.flags = DoRed | DoGreen | DoBlue;
	XStoreColor(dpy, cmap, &color);
/* DANA ECKART
	color_gc[index] = XCreateGC(dpy, mainWindow, 0, NULL);
        XSetForeground(dpy, color_gc[index], color.pixel);
	return index;
*/
	return color.pixel;
}

/* Set the values for Black_Pixel and White_Pixel. */
static void Black_and_White_Pixels(int max_colors) {
	if (max_colors <= 2) {
		Black_Pixel = BlackPixel(dpy, screen);
		White_Pixel = WhitePixel(dpy, screen);
	}
	else {
		/* Assume that the darkest pixels are given first and
		   the lightest in color are given last.  In either
		   case, both should be sufficiently different in color
		   to be readable.
		*/
		White_Pixel = pixels[max_colors-1];
		Black_Pixel = pixels[0];
	}
}

/* Do all the setup and housekeeping for starting an x11 window. */
void setup_x11(char *map_name, char *display, 
		int first_cell_side_x, int first_cell_side_y,
		char *title) {
	Visual *visual;
	XSizeHints sizehints;
	int max_x_pixels, max_y_pixels, depth;
	long int ncolors;
	char *default_file;

	x11_title = title;

	if (!(dpy = XOpenDisplay(display)))
	    	error("Can't open display '%s'", display);

	screen = XDefaultScreen(dpy);

	rootWindow = RootWindow(dpy, screen);

	/* Do setup for displaying the time in the time window. */
	font_info = XLoadQueryFont(dpy, FONT);
	if (!font_info)
		error("Unable to load font '%s'", FONT);
	charheight = font_info->ascent + font_info->descent;
	time_window_height = 1.5*charheight;

	/* Get the height and width of the screen in pixels. */
	max_x_pixels = XDisplayWidth(dpy, screen);
	max_y_pixels = XDisplayHeight(dpy, screen) - time_window_height - 1;

	cell_side_ratio = (float) first_cell_side_y / (float) first_cell_side_x;

	/* Determine the size of the sides of each displayed cell. */
	cell_side_x = (max_x_pixels / range[range_dim_1_index] > 
				max_y_pixels / range[range_dim_2_index] ?
			max_y_pixels / range[range_dim_2_index] : 
			max_x_pixels / range[range_dim_1_index]);

	if (cell_side_ratio < 1) {
		if (cell_side_x > MAX_CELL_SIDE) 
			cell_side_x = MAX_CELL_SIDE;

		/* Make sure cell_vals are "visible". */
		if (cell_side_x <= 0)
			error("Too many cells to display on screen", 
				(char*) NULL);

		cell_side_y = cell_side_x * cell_side_ratio;
	}
	else {
		cell_side_y = cell_side_x * cell_side_ratio;

		if (cell_side_y > MAX_CELL_SIDE) 
			cell_side_y = MAX_CELL_SIDE;

		/* Make sure cell_vals are "visible". */
		if (cell_side_y <= 0)
			error("Too many cells to display on screen", 
				(char*) NULL);

		cell_side_x = cell_side_y / cell_side_ratio;
	}

	/* Go with the prefered size if possible. */
	if (cell_side_x > first_cell_side_x) cell_side_x = first_cell_side_x;
	if (cell_side_y > first_cell_side_y) cell_side_y = first_cell_side_y;

	mainWidth = range[range_dim_1_index]*cell_side_x;
	mainHeight = range[range_dim_2_index]*cell_side_y + 
		      	time_window_height;

	/* Get the default display inforamtion. */
	visual = DefaultVisual(dpy, screen);
	depth =	DefaultDepth(dpy,screen);
        cmap = DefaultColormap(dpy, screen);

	/* If a map file was given, read it; otherwise use the default. */
	if (!map_name) {
		if (DisplayCells(dpy, screen) > 2) default_file = "c-map";
		else default_file = "bw-map";

		map_name = (char*) malloc((unsigned int) (strlen(LIB_DIR) + 
						strlen(default_file) + 2));
		if (!map_name)
			error("Out of memory", (char*) NULL);
		sprintf(map_name, "%s%s", LIB_DIR, default_file);
	}
	ncolors = file_size(map_name);

	/* Set up a color map. */
	if (DisplayCells(dpy, screen) > 2) {
		/* Create a new colormap if not a black/white display. */
		XVisualInfo     vinfo;
		int planes = DisplayPlanes(dpy,screen);
 		if (!(XMatchVisualInfo(dpy, screen, planes, PseudoColor, &vinfo)
	              || 
		      XMatchVisualInfo(dpy, screen, planes, GrayScale, &vinfo)
                      || 
		      XMatchVisualInfo(dpy, screen, planes, DirectColor, &vinfo)))
			error("Unable to set up the color mapping",
	 			(char*) NULL);

		visual = vinfo.visual;
		depth = vinfo.depth;

		/* Only create a new colormap, if the current one doesn't
		   have enough room in it for the map read in.
		*/
		if (! XAllocColorCells(dpy, cmap, False, NULL, 0, 
				       pixels, ncolors)) {
			cmap = XCreateColormap(dpy, rootWindow,
						visual, AllocNone);

			/* Get the color cells. */
			if (! XAllocColorCells(dpy, cmap, False, NULL, 0, 
					       pixels, ncolors))
				error("Unable to allocate %d colors", 
					(char*) ncolors);
		}
	}
	else if (ncolors > 2)
		error("This display can only show %d colors", 
			(char*) (long) DisplayCells(dpy, screen));

	fprintf(stderr, "Using %d colors...\n", ncolors);

	/* Create the palette main window and set its standard properties. */
	xswa.event_mask = KeyPressMask | StructureNotifyMask | ExposureMask
				| ButtonPressMask | VisibilityChangeMask;
	xswa.border_pixel = Black_Pixel;
	xswa.background_pixel = Black_Pixel;
	xswa.colormap = cmap;

	/* Configure the window size hints. */
	sizehints.flags = PMinSize | PSize;
	sizehints.x = 0;
	sizehints.y = 0;
	sizehints.min_width = range[range_dim_1_index];
	sizehints.min_height = range[range_dim_2_index] + 
						time_window_height;
	sizehints.width = mainWidth;
	sizehints.height = mainHeight;

	/* Create the window. */
	mainWindow = XCreateWindow(dpy, rootWindow,
			           sizehints.x, sizehints.y,
			           sizehints.width, sizehints.height,
			           1 /* Border Width */,
				   depth,
			           InputOutput,
			           visual,
			           CWEventMask | CWBackPixel | CWBorderPixel
					| CWColormap,
			           &xswa);

	if (!mainWindow) error("Unable to open a window", (char*) NULL);

	/* Define the window's name. */
	XSetStandardProperties(dpy, mainWindow, x11_title, x11_title,
			       None, NULL, 0, &sizehints);

	read_map(map_name, ncolors <= 2);
	Black_and_White_Pixels(ncolors);

        gc = XCreateGC(dpy, mainWindow, 0, NULL);
	cell_pixmap = XCreatePixmap(dpy, mainWindow,
				    range[range_dim_1_index]*cell_side_x,
				    range[range_dim_2_index]*cell_side_y,
				    depth);

        XSetFont(dpy, gc, font_info->fid);

	/* Get a more accurate looking cursor. */
	XDefineCursor(dpy, mainWindow,  XCreateFontCursor(dpy, XC_crosshair));

	/* Assign intial values for the cursor location to be used for
	   neighborhood value viewing within a sub-window.
	*/
	neighbor_x = range[range_dim_1_index]/2;
	if (range_dim_given > 1)
		neighbor_y = range[range_dim_2_index]/2;

	XSetInputFocus(dpy, rootWindow, RevertToPointerRoot, CurrentTime);
	XSelectInput(dpy, mainWindow, ExposureMask | StructureNotifyMask |
		     VisibilityChangeMask | ButtonPressMask | KeyPressMask);
	XMapWindow(dpy, mainWindow);

	/* Wait for the first event. */
	while (!XPending(dpy));
}

/* Start displaying the cell_vals. */
void display_x11(int field) {

	redisplay_cells();

	/* Display cell_vals for the rest of time. */
	while (1) {
		do_event();
#if !COMBINED
		if (next_time >= 0) 
#endif
		{
#if COMBINED
#if MAX_CPUS > 1
			cellang_main(output, view, filter, field, cpus-1);
#else
			cellang_main(output, view, filter, field);
#endif
#else
			read_cell_block();
#endif

#if COMBINED
			if (1 == report_time_step || 
			    _time_step == next_report_time || 
			    _time_step == next_read_time)
#endif
			{
				display_time();
				display_x11_cells();
			}

#if COMBINED
			if (_time_step == next_read_time)
				get_report_time(time_file);
			else if (report_time_step > 0 && _time_step >= next_report_time)
				next_report_time = _time_step + report_time_step;
			_time_step++;
#else
			if (next_time >= 0) _time_step = next_time;
#endif
	
			/* Pause times of 0 or less are ignored. */
			if (pause_time > 0) sleep((unsigned int) pause_time);
		}
	}
}

#endif
