% Bumble: a sample agent file for the Virtual World system
% Donald Nute
% 3/13/97

% Bumble just moves at random until he starves or is killed.

:- dynamic [agent_predicates/1,agent/1,tried/0,last/1,pushed/0].

agent_predicates([agent,opp]).

% Try to go through any nearby door if you didn't try last move.

agent(move(Dir)) :-
	\+ retract(tried),
	look(_,_,_,_,[_,NW,N,NE,_],[_,W,_,E,_],[_,SW,S,SE,_],_),
	member(p,[NW,N,NE,W,E,SW,S,SE],Num),
	assert(tried),
	member(Dir,[nw,n,ne,w,e,sw,s,se],Num).

% Consume any nearby consumable object.

agent(move(Dir)) :-
	look(_,_,_,_,[_,NW,N,NE,_],[_,W,_,E,_],[_,SW,S,SE,_],_),
	member(Obj,[NW,N,NE,W,E,SW,S,SE],Num),
	consumable(Obj),
	Obj \= o,
	member(Dir,[nw,n,ne,w,e,sw,s,se],Num),
	retractall(last(_)).

% Collect any nearby collectable object.

agent(move(Dir)) :-
	look(_,_,_,_,[_,NW,N,NE,_],[_,W,_,E,_],[_,SW,S,SE,_],_),
	member(Obj,[NW,N,NE,W,E,SW,S,SE],Num),
	collectable(Obj),
	member(Dir,[nw,n,ne,w,e,sw,s,se],Num),
	retractall(last(_)).

% Try pushing nearby inanimate objects

agent(move(Dir)) :-
	\+ retract(pushed),
	look(_,_,_,_,[_,NW,N,NE,_],[_,W,_,E,_],[_,SW,S,SE,_],_),
	member(Obj,[NW,N,NE,W,E,SW,S,SE],Num),
	\+ animate(Obj),
	\+ member(Obj,[w,o]),
	member(Dir,[nw,n,ne,w,e,sw,s,se],Num),
	assert(pushed).

% Move away from any animate object if posible

agent(move(Dir)) :-
	look(_,_,_,_,[_,NW,N,NE,_],[_,W,_,E,_],[_,SW,S,SE,_],_),
	member(Obj1,[NW,N,NE,W,E,SW,S,SE],Num1),
	animate(Obj1),
	member(BadDir,[nw,n,ne,w,e,sw,s,se],Num1),
	opp(BadDir,[List]),
	member(Dir,List),
	member(Dir,[nw,n,ne,w,e,sw,s,se],Num2),
	member(o,[NW,N,NE,W,E,SW,S,SE],Num2),
	retractall(last(_)).

% About 87% of the time, move in the direction you went 
% last if it is clear.

agent(move(Dir)) :-
	last(Dir),
	random_direction(X),
	member(X,[nw,n,ne,w,e,sw,s,se],Test),
	Test > 1,
	member(Dir,[nw,n,ne,w,e,sw,s,se],Num),
	look(_,_,_,_,[_,NW,N,NE,_],[_,W,_,E,_],[_,SW,S,SE,_],_),
	member(o,[NW,N,NE,W,E,SW,S,SE],Num).

% Move at random to empty space.

agent(move(Dir)) :-
	retractall(last(_)),
	look(_,_,_,_,[_,NW,N,NE,_],[_,W,_,E,_],[_,SW,S,SE,_],_),
	member(o,[NW,N,NE,W,E,SW,S,SE]),
	repeat,
	random_direction(Dir),
	member(Dir,[nw,n,ne,w,e,sw,s,se],Num2),
	member(o,[NW,N,NE,W,E,SW,S,SE],Num2),
	assert(last(Dir)).

% When all else fails, sit out a move.

agent(sit).


opp(n,[sw,s,se]).
opp(e,[nw,w,sw]).
opp(w,[se,e,sw]).
opp(s,[ne,n,nw]).
opp(se,[w,nw,n]).
opp(sw,[n,ne,e]).
opp(nw,[e,se,s]).
opp(ne,[s,sw,w]).
