% CASTLE.VW
% a Virtual World file
%
% Donald Nute
% 12/2/97
%
% Notes: 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%

:- dynamic vagent_data/5,vmap/3,vobject_icon/2,veff/3,vact/3,vkey/7,vcurrent_level/1.
:- dynamic movable/1,collectable/1,consumable/1,animate/1.

vagent_data(6,2,100,0,[gold]).

vobject_icon(f,fruit).		% Nourishing food
vobject_icon(c,rcross).		% Cure
vobject_icon(o,empty).		% Empty space
vobject_icon(p,door).		% Portal
vobject_icon(w,rbox).		% Solid wall
vobject_icon(grave,rip).

veff(f,25,0).
veff(c,-1,-100).
veff(o,-1,0).
veff(p,-1,0).
veff(w,-3,0).
veff(rb,-1,0).
veff(gb,-1,0).
veff(h2o,-1,0).
veff(jane,-1,0).
veff(throne,-1,0).
veff(h,-1,25).
veff(1,-1,0).
veff(2,-1,0).
veff(3,-1,0).

% If you push against a thorn, it will hurt you
% unless you have the shield.

veff(@,-1,X) :-
	vagent_data(_,_,_,_,Inv),
	(
	 member(shield,Inv) -> X is 0
	;
	 X is 10
	).

% If you push or strike the hag, she will hurt you
% badly unless you have the gold.

veff(hag,-1,X) :-
	vagent_data(_,_,_,_,Inv),
	(
	 member(gold,Inv) -> X is 0
	;
	 X is 25
	).

% If you push or strike the troll, he will hurt you VERY 
% badly unless you have the shield.

veff(troll,-1,X) :-
	vagent_data(_,_,_,_,Inv),
	(
	 member(shield,Inv) -> X is 0
	;
	 X is 50
	).

% If you push or strike the dragon, he will kill you.

veff(dragon,-1,100).

consumable(f).
consumable(o).

vobject_icon(a,knight).
vobject_icon(h,hornet).	
vobject_icon(s,snail).
vobject_icon(t,tree).
vobject_icon(rk,rkey).
vobject_icon(bk,bkey).
vobject_icon(gk,gkey).
vobject_icon(wk,wkey).
vobject_icon(pk,pkey).
vobject_icon(tk,tkey).
vobject_icon(mk,mkey).
vobject_icon(yk,ykey).
vobject_icon(gb,gbutton).	
vobject_icon(rb,rbutton).
vobject_icon(bx,bbox).	
vobject_icon(yx,ybox).
vobject_icon(gx,gbox).
vobject_icon(sx,sbox).
vobject_icon(px,pbox).
vobject_icon(jane,princess).
vobject_icon(hag,witch).
vobject_icon(spray,spray_can).
vobject_icon(shield,shield).
vobject_icon(throne,throne).
vobject_icon(sword,sword).
vobject_icon(dragon,dragon).
vobject_icon(@,flower2).
vobject_icon(troll,troll).
vobject_icon(bane,flower1).
vobject_icon(gold,coin).
vobject_icon(h2o,water).
vobject_icon(1,one).
vobject_icon(2,two).
vobject_icon(3,three).

collectable(rk).
collectable(bk).
collectable(gk).
collectable(wk).
collectable(pk).
collectable(tk).
collectable(mk).
collectable(yk).
collectable(sword).
collectable(shield).
collectable(spray).
collectable(bane).
collectable(gold).

vkey(none,1,9,11,2,9,2).
vkey(none,2,9,1,1,9,10).
vkey(none,2,1,3,3,16,9).
vkey(none,3,17,9,2,2,3).
vkey(none,2,1,9,4,16,3).
vkey(none,4,17,3,2,2,9).
vkey(none,2,10,11,5,10,2).
vkey(none,5,10,1,2,10,10).
vkey(none,5,2,7,5,2,8) :- 
	vmap(5,6,[_,gb|_]).
vkey(none,5,2,7,Floor,2,6) :- 
	vmap(5,6,[_,rb|_]),
	vmap(5,9,[_,N|_]),
	Floor is N + 4.
vkey(none,6,2,7,6,2,8) :- 
	vmap(6,6,[_,gb|_]).
vkey(none,6,2,7,Floor,2,6) :- 
	vmap(6,6,[_,rb|_]),
	vmap(5,9,[_,N|_]),
	Floor is N + 4.
vkey(none,7,2,7,7,2,8) :- 
	vmap(7,6,[_,gb|_]).
vkey(none,7,2,7,Floor,2,6) :- 
	vmap(7,6,[_,rb|_]),
	vmap(5,9,[_,N|_]),
	Floor is N + 4.


% Hornets will sting you mildly if given a chance. 
% If you push them, they will sting you badly.

animate(h).

vact(h,XX,YY) :-
	vlook(XX,YY,
		[A,B,C,D,E],
		[F,G,H,I,J],
		[K,L,M,N,O],
		[P,Q,R,S,T],
		[U,V,W,X,Y]),
	member(a,[G,H,I,L,N,Q,R,S]),
	!,
	retract(vagent_data(XXX,YYY,Str,Dmg,Inv)),
	NuDmg is Dmg + 3,
	assert(vagent_data(XXX,YYY,Str,NuDmg,Inv)),
	vstatus_report.

vact(h,X,Y) :-
	vrandom_permutation([n,s,e,w,ne,se,nw,sw],List),
	member(Dir,List),
	vtransform(X,Y,Dir,XX,YY),
	vinspect(current,XX,YY,o),
	!,
	vmove(h,X,Y,XX,YY).

% The troll will disappear if you attack him with the sword.
% If you stand beside the troll and you don't have the shield,
% he will trounce you!

animate(troll).

vreact(troll,X,Y) :-
	vagent_data(_,_,_,_,Inv),
	member(sword,Inv),
	vdraw(X,Y,o).

vact(troll,XX,YY) :-
	vlook(XX,YY,
		_,
		[_,A,B,C,_],
		[_,D,_,E,_],
		[_,F,G,H,_],
		_),
	member(a,[A,B,C,D,E,F,G,H]),
	vagent_data(_,_,_,_,Inv),
	\+ member(shield,Inv),
	retract(vagent_data(XXX,YYY,Str,Dmg,Inv)),
	NuDmg is Dmg + 20,
	assert(vagent_data(XXX,YYY,Str,NuDmg,Inv)),
	vstatus_report.

% If you get close to the dragon, he will toast you!
% If you have the dragon bane, he will run to the river.

animate(dragon).

vact(dragon,XX,YY) :-
	vlook(XX,YY,
		_,
		[_,A,B,C,_],
		[_,D,_,E,_],
		[_,F,G,H,_],
		_),
	member(a,[A,B,C,D,E,F,G,H]),
	retract(vagent_data(XXX,YYY,Str,Dmg,Inv)),
	NuDmg is Dmg + 50,
	assert(vagent_data(XXX,YYY,Str,NuDmg,Inv)),
	vstatus_report.

vact(dragon,XX,YY) :-
	vlook(XX,YY,
		[A,B,C,D,E],
		[F,G,H,I,J],
		[K,L,_,M,N],
		[O,P,Q,R,S],
		[T,U,V,W,X]),
	member(a,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X]),
	vagent_data(_,_,_,_,Inv),
	member(bane,Inv),
	(
	 L = o,
	 vtransform(XX,YY,w,XXX,YYY),
	 vmove(dragon,XX,YY,XXX,YYY)
	;
	 L = h2o,
	 vdraw(XX,YY,o)
	).

% The hag will disappear if you give her gold.

animate(hag).

vreact(hag,X,Y) :-
	vagent_data(_,_,_,_,Inv),
	member(gold,Inv),
	retract(vagent_data(A,B,C,D,Inv)),
	removeall(gold,Inv,NewInv),
	assert(vagent_data(A,B,C,D,NewInv)),
	vdraw(X,Y,o),
	vstatus_report.

% Princess Jane will follow you if the hag has left.
% She will move out of the way if you push her.
% She will die if she comes into contact with anything
% harmful (dragon, troll, hornet, thorn.)

animate(jane).
movable(jane).

vact(jane,XX,YY) :-
	\+ vactor(7,[hag,_,_]),
	vlook(XX,YY,
		[A,B,C,D,E],
		[F,G,H,I,J],
		[K,L,_,M,N],
		[O,P,Q,R,S],
		[T,U,V,W,X]),
	(
	 member(Actor,[G,H,I,L,M,P,Q,R]),
	 member(Actor,[dragon,troll,h,@]),
	 vdraw(XX,YY,grave),
	 retract(vagent_data(AX,AY,Str,_,Inv)),
	 assert(vagent_data(AX,AY,Str,100,Inv)),
	 vstatus_report,
	 assert(message(`Too bad! The princess was killed and you died of a broken heart!`))
	;
	 member(a,[G,H,I,L,M,P,Q,R])
	;
	 vchase_agent(jane,XX,YY,[A,B,C,D,E,F,J,K,N,O,S,T,U,V,W,X])
	).

vchase_agent(Actor,XX,YY,[A,B,C,D,E,F,J,K,N,O,S,T,U,V,W,X]) :-
	(
	 vchase_agent(Actor,XX,YY,[A,B,F],nw)
	;
	 vchase_agent(Actor,XX,YY,[B,C,D],n)
	;
	 vchase_agent(Actor,XX,YY,[D,E,J],ne)
	;
	 vchase_agent(Actor,XX,YY,[J,N,S],e)
	;
	 vchase_agent(Actor,XX,YY,[S,X,W],se)
	;
	 vchase_agent(Actor,XX,YY,[U,V,W],s)
	;
	 vchase_agent(Actor,XX,YY,[O,T,U],sw)
	;
	 vchase_agent(Actor,XX,YY,[F,K,O],w)
	).

vchase_agent(Actor,X,Y,List,Dir) :-
	member(a,List),
	vtransform(X,Y,Dir,XX,YY),
	vinspect(current,XX,YY,o),
	!,
	vmove(Actor,X,Y,XX,YY).

vact(jane,X,Y) :-
	\+ vactor(7,[hag,_,_]),
	integer_bound(0,XX,18),
	integer_bound(0,YY,12),
	vinspect(current,XX,YY,a),
	vlook(XX,YY,
		_,
		[_,G,H,I,_],
		[_,L,_,M,_],
		[_,P,Q,R,_],
		_),
	member(o,[G,H,I,L,M,P,Q,R],Pos),
	member(Dir,[nw,n,ne,w,e,sw,s,se],Pos),
	vtransform(XX,YY,Dir,XXX,YYY),
	vmove(jane,X,Y,XXX,YYY).

% The red elevator button turns green when pushed. It turns
% red again when the agent is not beside it.

animate(gb).

vreact(rb,1,6) :-
	vdraw(1,6,gb).
vact(gb,1,6) :-
 	vinspect(current,2,6,o),
 	vdraw(1,6,rb).

% The elevator floor indicator changes when pushed.

vreact(Number,1,9) :-
	member(Number,[1,2,3],Pos),
	member(NewNumber,[2,3,1],Pos),
	forall(member(Floor,[5,6,7]),
		(retract(vmap(Floor,9,[A,_,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R])),
		 assert(vmap(Floor,9,
			[A,NewNumber,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R])))),
	vdraw(1,9,NewNumber).

vact(_,_,_).

% Every level in every world should have walls two blocks
% thick on all sides. Otherwise, the agent and actors would
% be able to look beyond the edge of the world. This means
% that the maximum room size is 15X9 squares.

% The current level is set to 1 when the world file is loaded.

% Level 1: The Castle

%                              1 1 1 1 1 1 1 1 1
%          0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 
vmap(1, 0,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(1, 1,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(1, 2,[w,w,f,f,f,w,o,o,o,throne,o,o,o,w,o,o,o,w,w]).
vmap(1, 3,[w,w,f,f,f,w,o,o,o,o,o,o,o,w,o,bk,o,w,w]).
vmap(1, 4,[w,w,f,f,f,w,o,o,o,o,o,o,o,w,o,o,o,w,w]).
vmap(1, 5,[w,w,w,o,w,w,w,w,w,o,w,w,w,w,w,o,w,w,w]).
vmap(1, 6,[w,w,o,o,o,w,o,o,o,o,o,o,o,w,o,o,o,w,w]).
vmap(1, 7,[w,w,o,shield,o,w,o,o,o,o,o,o,o,w,o,o,o,w,w]).
vmap(1, 8,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(1, 9,[w,w,o,o,o,w,o,o,o,o,o,o,o,w,o,o,o,w,w]).
vmap(1,10,[w,w,w,w,w,w,w,w,w,o,w,w,w,w,w,w,w,w,w]).
vmap(1,11,[w,w,w,w,w,w,w,w,w,p,w,w,w,w,w,w,w,w,w]).
vmap(1,12,[w,w,w,w,w,w,w,w,w,o,w,w,w,w,w,w,w,w,w]).
%          0 1 2 3 4 5 6 7 8 9 1 1 1 1 1 1 1 1 1
%                              0 1 2 3 4 5 6 7 8 

% Level 2: The Field

%                              1 1 1 1 1 1 1
%              2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
vmap(2, 0,[w,w,w,w,w,w,w,w,w,o,w,w,w,w,w,w,w,w,w]).
vmap(2, 1,[w,w,w,w,w,w,w,w,w,p,w,w,w,w,w,w,w,w,w]).
vmap(2, 2,[w,w,w,w,w,w,w,w,w,o,w,w,w,w,w,w,w,w,w]).
vmap(2, 3,[o,p,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w,w]).
vmap(2, 4,[w,w,w,@,o,o,o,o,o,o,o,o,o,o,t,o,w,w,w]).
vmap(2, 5,[w,w,w,o,@,o,o,o,o,o,o,o,o,o,o,o,w,w,w]).
vmap(2, 6,[w,w,w,@,o,@,o,@,o,@,o,o,o,o,o,o,gx,gx,gx]).
vmap(2, 7,[w,w,w,o,@,o,@,o,@,o,@,o,o,o,o,o,o,p,o]).
vmap(2, 8,[w,w,w,@,o,@,o,@,o,@,o,@,o,o,o,o,gx,gx,gx]).
vmap(2, 9,[o,p,o,o,@,o,@,o,@,o,@,o,@,o,o,o,w,w,w]).
vmap(2,10,[w,w,w,w,w,w,w,w,w,w,o,w,w,w,w,w,w,w,w]).
vmap(2,11,[w,w,w,w,w,w,w,w,w,w,p,w,w,w,w,w,w,w,w]).
vmap(2,12,[w,w,w,w,w,w,w,w,w,w,o,w,w,w,w,w,w,w,w]).
%              2 3 4 5 6 7 8 9 1 1 1 1 1 1 1
%                              0 1 2 3 4 5 6 

% Level 3: The Dragon's Lair

%                              1 1 1 1 1 1 1
%              2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
vmap(3, 0,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(3, 1,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(3, 2,[w,w,o,o,o,h2o,h2o,sword,o,o,o,o,o,o,o,o,w,w,w]).
vmap(3, 3,[w,w,o,o,o,h2o,h2o,o,o,o,o,o,o,o,o,o,w,w,w]).
vmap(3, 4,[w,w,o,o,o,h2o,h2o,w,w,w,w,o,o,o,o,o,w,w,w]).
vmap(3, 5,[w,w,o,o,o,h2o,h2o,gold,o,o,w,o,o,o,o,o,w,w,w]).
vmap(3, 6,[w,w,o,o,o,h2o,h2o,o,o,o,o,dragon,o,o,o,o,w,w,w]).
vmap(3, 7,[w,w,o,o,o,h2o,h2o,o,o,o,w,o,o,o,o,o,w,w,w]).
vmap(3, 8,[w,w,o,o,o,h2o,h2o,w,w,w,w,o,o,o,o,o,w,w,w]).
vmap(3, 9,[w,w,o,o,o,h2o,h2o,o,o,o,o,o,o,t,o,o,o,p,o]).
vmap(3,10,[w,w,o,o,o,h2o,h2o,o,o,o,o,o,o,o,o,o,w,w,w]).
vmap(3,11,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(3,12,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
%              2 3 4 5 6 7 8 9 1 1 1 1 1 1 1
%                              0 1 2 3 4 5 6 

% Level 4: The Troll Bridge

%                              1 1 1 1 1 1 1
%              2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
vmap(4, 0,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(4, 1,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(4, 2,[w,w,o,o,w,h2o,h2o,w,@,o,@,bane,@,o,@,o,w,w,w]).
vmap(4, 3,[w,w,o,o,w,h2o,h2o,w,o,@,o,@,o,@,o,@,o,p,o]).
vmap(4, 4,[w,w,yk,o,w,h2o,h2o,w,@,o,@,o,@,o,@,o,w,w,w]).
vmap(4, 5,[w,w,o,o,w,w,w,w,o,@,o,@,o,@,o,@,w,w,w]).
vmap(4, 6,[w,w,o,o,o,o,troll,o,o,o,o,o,o,o,o,o,w,w,w]).
vmap(4, 7,[w,w,o,o,w,w,w,w,o,o,o,o,o,o,o,o,w,w,w]).
vmap(4, 8,[w,w,o,o,w,h2o,h2o,w,o,o,o,o,o,o,o,o,w,w,w]).
vmap(4, 9,[w,w,o,o,w,h2o,h2o,w,o,c,o,o,o,o,t,o,w,w,w]).
vmap(4,10,[w,w,o,o,w,h2o,h2o,w,o,o,o,o,o,o,o,o,w,w,w]).
vmap(4,11,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(4,12,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
%              2 3 4 5 6 7 8 9 1 1 1 1 1 1 1
%                              0 1 2 3 4 5 6 

% Level 5: The Tower, First Floor

%                              1 1 1 1 1 1 1
%              2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
vmap(5, 0,[w,w,w,w,w,w,w,w,w,w,o,w,w,w,w,w,w,w,w]).
vmap(5, 1,[w,w,w,w,w,w,w,w,w,w,p,w,w,w,w,w,w,w,w]).
vmap(5, 2,[w,w,w,w,w,w,w,w,w,w,o,w,w,w,w,w,w,w,w]).
vmap(5, 3,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w,w]).
vmap(5, 4,[w,w,w,w,w,w,w,w,w,w,w,o,o,o,o,o,w,w,w]).
vmap(5, 5,[w,w,w,w,w,w,w,w,w,w,w,o,o,o,o,o,w,w,w]).
vmap(5, 6,[w,rb,o,o,o,o,o,o,o,w,w,o,o,o,o,o,w,w,w]).
vmap(5, 7,[w,w,p,w,w,o,o,h,o,w,w,o,o,o,o,o,w,w,w]).
vmap(5, 8,[w,w,o,o,w,o,o,o,o,w,w,o,o,o,o,o,yx,yx,yx]).
vmap(5, 9,[w,1,o,o,w,o,h,o,o,w,w,o,o,o,o,o,o,p,o]).
vmap(5,10,[w,w,o,o,w,o,o,o,o,o,o,o,o,o,o,o,yx,yx,yx]).
vmap(5,11,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(5,12,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
%              2 3 4 5 6 7 8 9 1 1 1 1 1 1 1
%                              0 1 2 3 4 5 6 

% Level 6: The Tower, Second Floor

%                              1 1 1 1 1 1 1
%              2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
vmap(6, 0,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(6, 1,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(6, 2,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(6, 3,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(6, 4,[w,w,w,w,w,w,w,w,w,w,w,o,o,o,o,o,o,w,w]).
vmap(6, 5,[w,w,w,w,w,w,w,w,w,w,w,o,o,o,o,o,o,w,w]).
vmap(6, 6,[w,rb,o,o,o,o,o,o,o,w,w,o,o,o,o,o,o,w,w]).
vmap(6, 7,[w,w,p,w,w,o,o,o,o,w,w,o,o,o,o,o,o,w,w]).
vmap(6, 8,[w,w,o,o,w,o,o,o,o,w,w,o,o,o,o,o,o,w,w]).
vmap(6, 9,[w,1,o,o,w,o,o,spray,o,w,w,o,o,o,o,o,o,w,w]).
vmap(6,10,[w,w,o,o,w,o,o,o,o,w,w,o,o,o,o,o,o,w,w]).
vmap(6,11,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(6,12,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
%              2 3 4 5 6 7 8 9 1 1 1 1 1 1 1
%                              0 1 2 3 4 5 6 

% Level 7: The Tower, Third Floor

%                              1 1 1 1 1 1 1
%              2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
vmap(7, 0,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(7, 1,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(7, 2,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(7, 3,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(7, 4,[w,w,w,w,w,w,w,w,w,w,w,o,o,o,o,o,o,w,w]).
vmap(7, 5,[w,w,w,w,w,w,w,w,w,w,w,o,o,o,o,o,o,w,w]).
vmap(7, 6,[w,rb,o,o,o,o,o,o,o,w,w,o,o,o,o,o,o,w,w]).
vmap(7, 7,[w,w,p,w,w,w,o,o,o,w,w,o,o,o,o,o,o,w,w]).
vmap(7, 8,[w,w,o,o,o,w,o,jane,o,w,w,o,o,o,o,o,o,w,w]).
vmap(7, 9,[w,1,o,o,o,w,o,o,o,w,w,o,o,o,o,o,o,w,w]).
vmap(7,10,[w,w,o,o,o,w,o,hag,o,w,w,o,o,o,o,o,o,w,w]).
vmap(7,11,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(7,12,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
%              2 3 4 5 6 7 8 9 1 1 1 1 1 1 1
%                              0 1 2 3 4 5 6 

% Level 8: Lower Maze

%                              1 1 1 1 1 1 1
%              2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
vmap(8, 0,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(8, 1,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(8, 2,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(8, 3,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(8, 4,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(8, 5,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(8, 6,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(8, 7,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(8, 8,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(8, 9,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(8,10,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(8,11,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(8,12,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
%              2 3 4 5 6 7 8 9 1 1 1 1 1 1 1
%                              0 1 2 3 4 5 6 

% Level 9: Upper Maze

%                              1 1 1 1 1 1 1
%              2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
vmap(9, 0,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(9, 1,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(9, 2,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(9, 3,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(9, 4,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(9, 5,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(9, 6,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(9, 7,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(9, 8,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(9, 9,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(9,10,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(9,11,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(9,12,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
%              2 3 4 5 6 7 8 9 1 1 1 1 1 1 1
%                              0 1 2 3 4 5 6 

% Level 10

%                               1 1 1 1 1 1 1
%               2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
vmap(10, 0,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(10, 1,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(10, 2,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(10, 3,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(10, 4,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(10, 5,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(10, 6,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(10, 7,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(10, 8,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(10, 9,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(10,10,[w,w,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,w,w]).
vmap(10,11,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
vmap(10,12,[w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w]).
%               2 3 4 5 6 7 8 9 1 1 1 1 1 1 1
%                               0 1 2 3 4 5 6 

