% Virtual World (Vworld.pl)
%
% Donald Nute
% Department of Philosophy and the Artificial Intelligence Center
% The University of Georgia
% 12/1/97
%
% This program contains the driver for operating an artificial environment into
% which an artificial agent may inserted. To complete the environment, a world
% file must be loaded after this program is initiated using the vmain command.
% The file empty.pl contains a basic format for a world file, and hornet.pl
% is a sample world file complete with examples of simple actors. Also needed
% is the file vworld.nil, an icon library for the program. The program is written
% run under LPA WinProlog version 3.3 and was created for use in CS/PHY 865 -
% Logic and Logic Programming, spring quarter, 1997.

:- dynamic vagent_data/5,vmap/3,vobject_icon/2,veff/3,vact/3,vreact/3.
:- dynamic key/7,vcurrent_level/1,vstop/0,vstep/1,vrelocate_agent/0.
:- dynamic movable/1,collectable/1,consumable/1,animate/1.
:- dynamic agent/1,agent_predicates/1.

% vagent_icons(+List) stores a list of available icons for the agent.

vagent_icons([man2,knight,princess,billcat,witch,troll,puppy,man1]).

% vstartup is the initial query when the executable version of
% Virtual World is created.

vstartup :-
	sttbox('',  -1),
	catch(Error,vmain),
	error_message(Error,Message),
	(
	 Error = 0
	;
	 msg_box('Error',Message,0,_)
	),
	halt.

% vmain creates the Virtual World window and menus, starts the
% Virtual World window handler, and closes the window and menus
% when the dialog ends. 

vmain :-
	prolog_flag(unknown,_,fail),
	wdcreate(vworld,`Virtual World`,0,0,640,480,
		[ws_popup,ws_sysmenu,ws_caption,ws_border]),
	wccreate((vworld,2),static,`Strength: `,3,3,47,20,
		[ws_child,ws_visible,ss_right]),
	wccreate((vworld,200),static,``,51,3,18,15,
		[ws_child,ws_visible,ss_left]),
	wccreate((vworld,3),static,`Damage: `,72,3,45,20,
		[ws_child,ws_visible,ss_right]),
	wccreate((vworld,300),static,``,119,3,18,15,
		[ws_child,ws_visible,ss_left]),
	wccreate((vworld,4),static,`Inventory: `,219,3,50,20,
		[ws_child,ws_visible,ss_right]),
	wccreate((vworld,400),static,``,273,3,240,15,
		[ws_child,ws_visible,ss_left]),
	wccreate((vworld,5),static,`Moves: `,140,3,45,20,
		[ws_child,ws_visible,ss_right]),
	wccreate((vworld,500),static,``,186,3,25,15,
		[ws_child,ws_visible,ss_left]),
	wccreate((vworld,6),static,`Last Move: `,516,3,60,20,
		[ws_child,ws_visible,ss_right]),
	wccreate((vworld,600),static,``,580,3,220,15,
		[ws_child,ws_visible,ss_left]),
	wccreate((vworld,1),grafix,``,15,20,610,418,
		[ws_child,ws_visible,ws_border]),
	vload_icons,
	vmake_menus,
	window_handler(vworld,v_hndl),
	call_dialog(vworld,ok),
	wmclose(vfile),
	wmclose(vtest),
	wmclose(vactions),
	wmclose(voptions),
	wmclose(vhelp),
	wclose(vworld),
	help('vworld.hlp',2,0).

vmain.

% vrun starts the action in Virtual World and keeps it going until
% the File/Stop menu selection is made, the agent dies, or an error
% is encountered in the agent code.

vrun :-
	repeat,
	catch(Code,(test(Goal),Goal,!)),
	vrun(Code,Goal),
	vhalt.

vrun(0,Goal) :-
	veverybody_act,
	vlast(Goal),
	(
	 vwell_and_able(_,_,_,_,_)
	;
	 assert(vstop)
	),
	!.

vrun(Code,Goal) :-
	vlast(Goal),
	error_message(Code,Message),
	msgbox(`Agent code error`,Message,16,_),
	assert(vstop).

test(Move) :-
	agent(Move).

% vhalt checks to see if one of the conditions for halting the
% action in Virtual World is satisfied.

vhalt :- retract(vstop).

vhalt :- 
	vstep(N),
	vnumber_of_moves(M),
	M >= N,
	retractall(vstop).

% vmake_menus creates the menus for Virtual World and
% puts them on the menu bar.

vmake_menus :-
	wmcreate(vfile),
	vmnuadd(vworld,-1,`&File`,vfile),
	wmnuadd(vfile,-1,`Load &World`,2000),
	wmnuadd(vfile,-1,`&Load Agent`,2001),
	wmnuadd(vfile,-1,`&Save Agent`,2002),
	wmnuadd(vfile,-1,``,0),
	wmnuadd(vfile,-1,`Exit`,2003),
	wmnunbl(vfile,1,0),
	wmnunbl(vfile,2,0),

	wmcreate(vactions),
	vmnuadd(vworld,-1,`&Actions`,vactions),
	wmnuadd(vactions,-1,`&Run`,4000),
	wmnuadd(vactions,-1,`&Stop`,4001),
	wmnuadd(vactions,-1,`s&Tep`,4002),
	wmnunbl(vactions,0,0),
	wmnunbl(vactions,1,0),
	wmnunbl(vactions,2,0),
	
	wmcreate(vtest),
	vmnuadd(vworld,-1,`&Test`,vtest),
	wmnuadd(vtest,-1,`&North`,3004),
	wmnuadd(vtest,-1,`&South`,3003),
	wmnuadd(vtest,-1,`&East`,3002),
	wmnuadd(vtest,-1,`&West`,3001),
	wmnuadd(vtest,-1,`s&It`,3000),
	wmnunbl(vtest,0,0),
	wmnunbl(vtest,1,0),
	wmnunbl(vtest,2,0),
	wmnunbl(vtest,3,0),
	wmnunbl(vtest,4,0),

	wmcreate(voptions),
	vmnuadd(vworld,-1,`&Options`,voptions),
	wmnuadd(voptions,-1,`Change &Icon`,5000),
	wmnuadd(voptions,-1,`Change &Location`,5001),
	wmnuadd(voptions,-1,`Reset &Strength`,5002),
	wmnuadd(voptions,-1,`Reset &Damage`,5003),
	wmnunbl(voptions,0,0),
	wmnunbl(voptions,1,0),
	wmnunbl(voptions,2,0),
	wmnunbl(voptions,3,0),

	wmcreate(vhelp),
	vmnuadd(vworld,-1,`&Help`,vhelp),
	wmnuadd(vhelp,-1,`&Contents`,6000),
	wmnuadd(vhelp,-1,``,0),
	wmnuadd(vhelp,-1,`&About`,6001).


% look(-S,-D,-I,-R1,-R2,-R3,-R4,-R5) returns the strength, damage, and 
% inventory of the agent plus the 5X5 grid of objects occupying 
% the locations in the current level of the world map centered on the
% agents current location.

look(Strength,Damage,Inventory,R1,R2,R3,R4,R5) :-
	vagent_data(XX,YY,Strength,Damage,Inventory),
	vlook(XX,YY,R1,R2,R3,R4,R5).

% vlook(+X,+Y,-R1,-R2,-R3,-R4,-R5) returns the 5X5 grid of objects 
% occupying the locations in the current level of the world map
% centered on the location X,Y.

vlook(XX,YY,
	[A,B,C,D,E],
	[F,G,H,I,J],
	[K,L,M,N,O],
	[P,Q,R,S,T],
	[U,V,W,X,Y]) :-
	Y1 is YY - 2,
	Y2 is YY - 1,
	Y4 is YY + 1,
	Y5 is YY + 2,
	IX is XX - 2,
	vcurrent_level(Lev),
	vscan(Lev,Y1,IX,[A,B,C,D,E]),
	vscan(Lev,Y2,IX,[F,G,H,I,J]),
	vscan(Lev,YY,IX,[K,L,M,N,O]),
	vscan(Lev,Y4,IX,[P,Q,R,S,T]),
	vscan(Lev,Y5,IX,[U,V,W,X,Y]).

% sit decrements the agent's strength and increments the move
% counter. This is an action an agent can perform.

sit :-
	retract(vagent_data(X,Y,S,D,I)),
	SS is S - 1,
	assert(vagent_data(X,Y,SS,D,I)).
	
% move(+Dir) tries to move the agent in the direction Dir. What
% happens depends on what is in direction Dir.

move(Dir) :-
	vwell_and_able(X,Y,Str,Dmg,Inv),
	!,
	vtransform(X,Y,Dir,XX,YY),
	vinspect(current,XX,YY,Obj),
	vpush(a,X,Y,Obj,XX,YY,Dir,NuX,NuY,AdjStr,AdjDmg,Item),
	NuStr is Str + AdjStr,
	NuDmg is Dmg + AdjDmg,
	vadd_to_inventory(Item,Inv,NuInv),
	retract(vagent_data(_,_,_,_,_)),
	assert(vagent_data(NuX,NuY,NuStr,NuDmg,NuInv)).

move(_).

% vmove(+Actor,+X,+Y,+XX,+YY) moves an Actor from location X,Y to
% location XX,YY in the current level, leaving location X,Y empty. 

vmove(Actor,X,Y,XX,YY) :-
	vdraw(X,Y,o),
	vdraw(XX,YY,Actor).

% vactor(+L,[-Actor,-X,-Y]) returns a list containing an actor other than 
% the agent in level L and its location X,Y.

vactor(L,[Actor,X,Y]) :-
	integer_bound(0,X,18),
	integer_bound(0,Y,12),
	vinspect(L,X,Y,Actor),
	animate(Actor).

% veverybody_act calls upon every actor in the current level other than the
% agent to act in a random order. After all actors have acted, the status
% display is updated.

veverybody_act :-
	vcurrent_level(L),
	findall(X,vactor(L,X),List),
	vrandom_permutation(List,ActorList),
	vtake_turns(ActorList),
	vstatus_report.

% vtake_turns(ActorList) calls upon the actors in the current level other
% than the agent to take turns acting in random order.

vtake_turns([]).

vtake_turns([[Actor,X,Y]|Rest]) :-
	vact(Actor,X,Y),
	!,
	vtake_turns(Rest).

% vaction(+Actor,+X,+Y) calls upon vact/3, which must be defined in the
% world file. If there is not information in the world file about how
% Actor is supposed to act, vaction/3 succeeds without doing anything.

vaction(Actor,X,Y) :-
	vact(Actor,X,Y).

vaction(_,_,_).

% vpush(+Actor,+X,+Y,+Obj,+XX,+YY,+Dir,-NuX,-NuY,-AdjStr,-AdjDmg,-Item)
% updates the world map and display that results when an Actor at
% location X,Y in the current level tries to move into the space occupied
% by an Obj at location XX,YY. push/12 also returns the new location 
% NuX,NuY of the Actor after the action, the changes AdjStr and AdjDmg
% to the Actor's strength and damage resulting from the action, and
% any Item that is to be added to the Actor's inventory as a result of
% the action.

% When an actor moves into the space of a consumable object, the
% object disappears and the actor enjoys/suffers the effects.

vpush(Actor,X,Y,Obj,XX,YY,_,XX,YY,AdjStr,AdjDmg,[]) :-
	consumable(Obj),
	veffect(Obj,AdjStr,AdjDmg),
	vmove(Actor,X,Y,XX,YY).

% When an actor moves into the space of a collectable object, the
% object disappears from the world map, is added to the actor's 
% inventory, and the actor enjoys/suffers the effects.

vpush(Actor,X,Y,Obj,XX,YY,_,XX,YY,AdjStr,AdjDmg,[Obj]) :-
	collectable(Obj),
	veffect(Obj,AdjStr,AdjDmg),
	vmove(Actor,X,Y,XX,YY).

% When an actor tries to move into the space of a movable object,
% that object moves out of the way unless this would require it
% to move into the space occupied by another object. In that case,
% the second object must also move. Thus, several objects may be
% moved unless some immovable object stands in the way. The effects
% of moving several objects with one push are accumulated and passed
% back to the actor. This particular clause is recursive since a
% movable object becomes an actor which may move another object.

vpush(Actor,X,Y,Obj,XX,YY,Dir,XX,YY,AdjStr,AdjDmg,[]) :-
	movable(Obj),
	vtransform(XX,YY,Dir,XXX,YYY),
	vinspect(current,XXX,YYY,NuObj),
	vpush(Obj,XX,YY,NuObj,XXX,YYY,Dir,XXX,YYY,AdjStr1,AdjDmg1,_),
	veffect(Obj,AdjStr2,AdjDmg2),
	AdjStr is AdjStr1 + AdjStr2,
	AdjDmg is AdjDmg1 + AdjDmg2,
	vmove(Actor,X,Y,XX,YY).

% To enter a portal, an agent must have any key required by the
% portal in its inventory. Portals are not simple doors. They can
% teleport an agent to any location within the world map. Keys
% and portal coordinates must be specified in the world file. If
% an agent lacks a key for a portal, the portal becomes an immovable
% object. Some portals may not require keys. Others may be locked,
% but require some other action to open them rather than possession
% of a key. Notice that a portal can become "blocked" if some object
% occupies its destination coordinates. Actors other than the agent
% cannot use portals.

vpush(a,X,Y,p,XX,YY,_,NuX,NuY,-1,0,[]) :-
	vcurrent_level(L),
	vkey(K,L,XX,YY,L2,NuX,NuY),
	vagent_data(_,_,_,_,Inv),
	(
	 K = none
	;
	 member(K,Inv)
	),
	vinspect(L2,NuX,NuY,o),
	!,
	vdraw(X,Y,o),
	retract(vcurrent_level(_)),
	assert(vcurrent_level(L2)),
	vdraw(NuX,NuY,a),
	vcolor_world.

% If none of the above apply, the agent is pushing against an
% immovable object. The agent does not change position, but
% the action may have far-ranging effects including changes in
% the agent's strength, damage, or inventory, the unlocking of
% a portal somewhere, or most anything else one can imagine.

vpush(Agent,X,Y,Obj,XX,YY,_,X,Y,AdjStr,AdjDmg,[]) :-
	vreaction(Obj,XX,YY),
	veffect(Obj,AdjStr,AdjDmg).

% vadd_to_inventory(+NewItem,+OldInventory,-NewInventory) appends
% whatever item an agent has just picked up (which may be none - 
% then NewItem = []) to the agent's inventory.

vadd_to_inventory([X],I,[X|I]).
vadd_to_inventory(_,I,I).

% vscan(+L,+Y,+X,-View) returns a list of the object at level L
% and position X,Y, plus the four objects occupying positions
% to the right of the first object. This routine is called when
% an actor looks at its surroundings.

vscan(L,Y,X,View) :-
	vmap(L,Y,List),
	vscan_aux(X,List,View).

vscan_aux(0,[A,B,C,D,E|_],[A,B,C,D,E]).
vscan_aux(X,[_|List],View) :-
	XX is X - 1,
	vscan_aux(XX,List,View).

% vinspect(+L,+X,+Y,-Obj) returns the object Obj located at 
% X,Y in level L of the currently loaded world map. L may
% be the atom current, in which case the current level is used, or it
% may be an integer representing an explicit level in the world map.

vinspect(current,X,Y,Obj) :-
	vcurrent_level(L),
	!,
	vmap(L,Y,List),
	vinspect_aux(X,List,Obj).

vinspect(L,X,Y,Obj) :-
	vmap(L,Y,List),
	vinspect_aux(X,List,Obj).

vinspect_aux(0,[Obj|_],Obj).
vinspect_aux(X,[_|List],Obj) :-
	X > 0,
	XX is X - 1,
	vinspect_aux(XX,List,Obj).

% vdraw(+X,+Y,+Obj) places the symbol for Obj in the world map at
% location X,Y in the current level. It also draws the icon for Obj
% on the screen at the X,Y location. rewrite/3 uses rewrite/5 to
% change the world map. rewrite/5 is a look-up table for fast
% processing.

vdraw(X,Y,Obj) :-
	vobject_icon(Obj,Icon),
	vcurrent_level(Lev),
	vdraw(Lev,X,Y,Obj),
	XX is X * 32,
	YY is Y * 32,
	gfx_begin((vworld,1)),
	gfx(icon(XX,YY,Icon)),
	gfx_end((vworld,1)),
	retractall(vrelocate_agent).

vdraw(X,Y,obj(_,Obj,_)) :-
	vobject_icon(Obj,Icon),
	vcurrent_level(Lev),
	vdraw(Lev,X,Y,Obj),
	XX is X * 32,
	YY is Y * 32,
	gfx_begin((vworld,1)),
	gfx(icon(XX,YY,Icon)),
	gfx_end((vworld,1)),
	retractall(vrelocate_agent).

% vdraw(+Lev,+X,+Y,+Obj) places Obj in the world map
% at location X,Y on level Lev. It does not draw any icons.

vdraw(Lev,0,Y,Obj) :-
	retract(vmap(Lev,Y,[_,A,B,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[Obj,A,B,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S])).
vdraw(Lev,1,Y,Obj) :-
	retract(vmap(Lev,Y,[A,_,B,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,Obj,B,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S])).
vdraw(Lev,2,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,_,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,Obj,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S])).
vdraw(Lev,3,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,_,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,Obj,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S])).
vdraw(Lev,4,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,_,F,G,H,I,J,K,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,Obj,F,G,H,I,J,K,L,M,N,O,P,Q,R,S])).
vdraw(Lev,5,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,_,G,H,I,J,K,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,Obj,G,H,I,J,K,L,M,N,O,P,Q,R,S])).
vdraw(Lev,6,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,_,H,I,J,K,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,Obj,H,I,J,K,L,M,N,O,P,Q,R,S])).
vdraw(Lev,7,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,_,I,J,K,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,Obj,I,J,K,L,M,N,O,P,Q,R,S])).
vdraw(Lev,8,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,_,J,K,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,Obj,J,K,L,M,N,O,P,Q,R,S])).
vdraw(Lev,9,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,_,K,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,Obj,K,L,M,N,O,P,Q,R,S])).
vdraw(Lev,10,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,_,L,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,Obj,L,M,N,O,P,Q,R,S])).
vdraw(Lev,11,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,_,M,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,Obj,M,N,O,P,Q,R,S])).
vdraw(Lev,12,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,_,N,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,Obj,N,O,P,Q,R,S])).
vdraw(Lev,13,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,_,O,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,Obj,O,P,Q,R,S])).
vdraw(Lev,14,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,_,P,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,Obj,P,Q,R,S])).
vdraw(Lev,15,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,_,Q,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,Obj,Q,R,S])).
vdraw(Lev,16,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,_,R,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Obj,R,S])).
vdraw(Lev,17,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,R,_,S])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,R,Obj,S])).
vdraw(Lev,18,Y,Obj) :-
	retract(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,R,S,_])),
	assert(vmap(Lev,Y,[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,R,S,Obj])).

% vtransform(+X,+Y,+Dir,-XX,-YY) determines the XX,YY coordinates of the
% location one square away from the location X,Y in direction DIR.

vtransform(X,Y,n,X,YY) :-
	YY is Y - 1.
vtransform(X,Y,s,X,YY) :-
	YY is Y + 1.
vtransform(X,Y,e,XX,Y) :-
	XX is X + 1.
vtransform(X,Y,w,XX,Y) :-
	XX is X - 1.
vtransform(X,Y,ne,XX,YY) :-
	XX is X + 1,
	YY is Y - 1.
vtransform(X,Y,nw,XX,YY) :-
	XX is X - 1,
	YY is Y - 1.
vtransform(X,Y,se,XX,YY) :-
	XX is X + 1,
	YY is Y + 1.
vtransform(X,Y,sw,XX,YY) :-
	XX is X - 1,
	YY is Y + 1.

% vstatus_report displays the current strength,
% damage, and inventory for the agent at the top
% of the screen. Strength is not allowed to exceed
% a maximum value of 300, and damage is not allowed
% to drop below a minimum of 0. vstatus_report also
% increments the move counter and displays it.

vstatus_report :-
	retract(vagent_data(X,Y,S,D,I)),
	(
	 S > 300 -> SS = 300
	;
	 SS = S
	),
	(
	 D < 0 -> DD = 0
	;
	 DD = D
	),
	assert(vagent_data(X,Y,SS,DD,I)),
	number_string(SS,SStr),
	wtext((vworld,200),SStr),
	number_string(DD,DStr),
	wtext((vworld,300),DStr),
	(
	 (I = [] -> II = ``)
	;
	 (
	  vcat(I,'',Atom),
	  atom_string(Atom,II)
	 )
	),
	wtext((vworld,400),II),
	retract(vnumber_of_moves(M)),
	MM is M + 1,
	assert(vnumber_of_moves(MM)),
	number_string(MM,MStr),
	wtext((vworld,500),MStr),
	vdie.	

% vwell_and_able(-X,-Y,-S,-D,-I) returns the location, strength,
% damage, and inventory for the agent provided the agent is
% "alive", that is, the agent has strength greater than 0 and
% damage less than 100. If the agent is not "alive", well_and_able/5
% fails.

vwell_and_able(X,Y,S,D,I) :-
	vagent_data(X,Y,S,D,I),
	S > 0,
	D < 100.

% vdie will replace the agent's icon on the screen with a tombstone
% if the agent "starves" (strength is below 0) or is "killed" (damage
% is above 100).

vdie :-
	\+ vwell_and_able(_,_,_,_,_),
	!,
	vagent_data(X,Y,_,_,_),
	XX is X * 32,
	YY is Y * 32,
	gfx_begin((vworld,1)),
	gfx(icon(XX,YY,rip)),
	gfx_end((vworld,1)),
	% Disable Actions, Test, and Options menu items.
	wmnunbl(vtest,0,0),
	wmnunbl(vtest,1,0),
	wmnunbl(vtest,2,0),
	wmnunbl(vtest,3,0),
	wmnunbl(vtest,4,0),
	wmnunbl(vactions,0,0),
	wmnunbl(vactions,1,0),
	wmnunbl(vactions,2,0),
	wmnunbl(voptions,0,0),
	wmnunbl(voptions,1,0).

vdie.

% vlast(+Action) displays the last action taken by the agent.

vlast(Action) :-
	vlast_string(Action,String),
	wtext((vworld,600),String).

vlast_string(sit,`sit`).
vlast_string(move(n),`move(n)`).
vlast_string(move(s),`move(s)`).
vlast_string(move(e),`move(e)`).
vlast_string(move(w),`move(w)`).
vlast_string(move(ne),`move(ne)`).
vlast_string(move(nw),`move(nw)`).
vlast_string(move(se),`move(se)`).
vlast_string(move(sw),`move(sw)`).
vlast_string(_,`error`).

% vload_icons loads the icons used to represent the objects
% and actors in vworld. All icons are stored in the icon
% library wvorld.nil.

vload_icons :-
	forall((member(Icon,
       [empty,
	dark,
	bbox,
	ybox,
	rbox,
	gbox,
	sbox,
	pbox,
	rkey,
	bkey,
	ykey,
	gkey,
	pkey,
	mkey,
	tkey,
	wkey,
	man1,
	man2,
	rip,
	door,
	fruit,
	hornet,
	puppy,
	snail,
	rbutton,
	gbutton,
	rcross,
	gcross,
	tree,
	sun,
	billcat,
	one,
	two,
	three,
	water,
	flower1,
	flower2,
	throne,
	spray_can,
	shield,
	sword,
	coin,
	knight,
	princess,
	dragon,
	troll,
	witch
	],
	M),
	N is M - 1),
	gfx_icon_load(Icon,'vworld.nil',N)).
	
% vcolor_world reads the world map for the current level
% and displays it.

vcolor_world :-
	vcurrent_level(Lev),
	vcolor_row(Lev,0).

vcolor_row(_,13).
vcolor_row(Lev,N) :-
	vmap(Lev,N,List),
	vcolor_cells(N,0,List),
	M is N + 1,
	vcolor_row(Lev,M).

vcolor_cells(N,19,_) :-
	X is N * 32,
	gfx_begin((vworld,1)),
	gfx(icon(X,608,rbox)),
	gfx_end((vworld,1)).
	
vcolor_cells(N,K,[Obj|Rest]) :-
	vobject_icon(Obj,Icon),
	!,
	X is 32 * K,
	Y is 32 * N,
	gfx_begin((vworld,1)),
	gfx(icon(X,Y,Icon)),
	gfx_end((vworld,1)),
	J is K + 1,
	vcolor_cells(N,J,Rest).

vcolor_cells(N,K,[obj(_,Obj,_)|Rest]) :-
	vobject_icon(Obj,Icon),
	!,
	X is 32 * K,
	Y is 32 * N,
	gfx_begin((vworld,1)),
	gfx(icon(X,Y,Icon)),
	gfx_end((vworld,1)),
	J is K + 1,
	vcolor_cells(N,J,Rest).

vcolor_cells(N,K,[_|Rest]) :-
	M is K + 1,
	vcolor_cells(N,M,Rest).

% veffect(+Obj,-S,-D) returns the change in strength and damage that will
% result if the agent pushes, picks up, or consumes Obj. This predicate
% uses the eff/3 predicate from the world file that is loaded.

veffect(Obj,S,D) :-
	veff(Obj,S,D),
	!.
veffect(_,0,0).

% vreaction(+Obj,+X,+Y) accomplishes the reaction of any immovable
% object when the agent "pushes" on it (tries to move into the space
% it occupies.) vreaction/3 calls the predicate vreact/3 which must be
% defined within the world file. If vreact/3 is not defined for the
% object and location, then there is no reaction. To play fair, multiple
% occurrences of the same type of object at different locations should
% behave in similar (but not necessarily identical ways. For example,
% "door openers" should all open a door somewhere, although "door openers"
% might not all unlock the same door.

vreaction(Obj,X,Y) :-
	vreact(Obj,X,Y),
	!.

vreaction(_,_,_).

% Randomization routines.

vrandom(N,M,K) :-
	J is M - N + 1,
	K is rand(J)//1 + N.

vrandom_member(X,List) :-
	len(List,L),
	vrandom(1,L,K),
	member(X,List,K).

vrandom_permutation(Old,New) :-
	vrandom_permutation(Old,[],New),
	!.

vrandom_permutation([],X,X).
vrandom_permutation(Old,SoFar,New) :-
	vrandom_member(Y,Old),
	remove(Y,Old,Next),
	!,
	vrandom_permutation(Next,[Y|SoFar],New).

random_direction(Dir) :-
	vrandom_member(Dir,[n,s,e,w,ne,se,nw,sw]).

% v_hndl/4 is the window handler for Virtual World.

v_hndl(vworld,msg_close,_,_) :-
	agent_predicates(_),
	msgbox(`Save Agent?`,
		`Do you want to save your agent?`,36,B),
	B = 6,
	vsave_agent,
	fail.

v_hndl(vworld,msg_close,_,ok).

% Instructions for File/Load World

v_hndl(vworld,msg_menu,2000,_) :-
	vload_world.

vload_world :-
	agent_predicates(_),
	msgbox(`Save Agent?`,
		`Do you want to save your agent?`,36,B),
	B = 6,
	vsave_agent,
	fail.

vload_world :-
	dirbox(`Load world file`,``,'*.vw',File),
	retractall(vmap(_,_,_)),
	reconsult(File),
	verify_world_loaded.

verify_world_loaded :-
	\+ vmap(_,_,_),
	!,
	msgbox(`Incorrect File`,
`Your world file did not contain a world map.`,16,_).

verify_world_loaded :-
	retractall(vcurrent_level(_)),
	assert(vcurrent_level(1)),
	retractall(vnumber_of_moves(_)),
	assert(vnumber_of_moves(-1)),
	vcolor_world,
	vstatus_report,
	% Disable Test, Actions, and Options menu items until agent is loaded.
	wmnunbl(vtest,0,0),
	wmnunbl(vtest,1,0),
	wmnunbl(vtest,2,0),
	wmnunbl(vtest,3,0),
	wmnunbl(vtest,4,0),
	wmnunbl(vactions,0,0),
	wmnunbl(vactions,1,0),
	wmnunbl(vactions,2,0),
	wmnunbl(voptions,0,0),
	wmnunbl(voptions,1,0),
	wmnunbl(voptions,2,0),
	wmnunbl(voptions,3,0),
	% Enable Load Agent and disable Save Agent options on Files menu.
	wmnunbl(vfile,1,1),
	wmnunbl(vfile,2,0).

% Instructions for File/Load Agent

v_hndl(vworld,msg_menu,2001,_) :-
	vload_agent.

vload_agent :-
	dirbox(`Load agent file`,``,'*.pl',File),
	retractall(agent(_)),
	reconsult(File),
	verify_agent_loaded.

verify_agent_loaded :-
	\+ clause(agent(_),_),
	msgbox(`Improper agent file.`,
		`The file you loaded did not contain any agent/1 clauses.`,
		16,_),
	!.

verify_agent_loaded :-
	% Enable Test, Actions, and Options menu items.
	wmnunbl(vtest,0,1),
	wmnunbl(vtest,1,1),
	wmnunbl(vtest,2,1),
	wmnunbl(vtest,3,1),
	wmnunbl(vtest,4,1),
	wmnunbl(vactions,0,1),
	wmnunbl(vactions,1,1),
	wmnunbl(vactions,2,1),
	wmnunbl(voptions,0,1),
	wmnunbl(voptions,1,1),
	wmnunbl(voptions,2,1),
	wmnunbl(voptions,3,1),
	% Enable Save Agent and disable Load Agent options in File menu.
	wmnunbl(vfile,1,0),
	wmnunbl(vfile,2,1),
	% Make the agent visible.
	vagent_data(X,Y,_,_,_),
	retractall(vobject_icon(a,_)),
	vagent_icons([Icon|_]),
	assert(vobject_icon(a,Icon)),
	vdraw(X,Y,a).

% Instructions for File/Save Agent

v_hndl(vworld,msg_menu,2002,_) :-
	vsave_agent.

vsave_agent :-
	\+ agent_predicates(_),
	!,
	msgbox(`Can't save agent.`,
		`There is no list of agent predicates to be saved.`,
		16,_).

vsave_agent :-
	dirbox(`Save Agent`,
	`Enter a new name for your agent file.`,'*.pl',File),
	agent_predicates(List),
	tell(File),
	forall(member(Predicate,List),listing(Predicate)),
	told.

vsave_agent.

% Instructions for File/Exit

v_hndl(vworld,msg_menu,2003,_) :-
	agent_predicates(_),
	msgbox(`Save Agent?`,
		`Do you want to save your agent?`,36,B),
	B = 6,
	vsave_agent,
	fail.

v_hndl(vworld,msg_menu,2003,ok) :-
	retractall(agent_predicates(_)),
	retractall(vmap(_,_,_)).

% Instructions for Actions/Run

v_hndl(vworld,msg_menu,4000,_) :-
	vrun.

% Instructions for Actions/Stop

v_hndl(vworld,msg_menu,4001,_) :-
	vset_stop.

vset_stop :-
	wdcreate(stop_dialog,`Set stop point`,155,150,300,120,
		[ws_popup,ws_caption,ws_border]),
	wccreate((stop_dialog,1),static,
`Number of moves at which action should stop: `,
		10,20,220,20,
		[ws_child,ws_visible,ss_right]),
	(
	 (
	  vstep(N),
	  number_string(N,NStr)
	 )
	;
	 NStr = `0`
	),
	wccreate((stop_dialog,2),edit,NStr,240,20,40,20,
		[ws_child,ws_visible,ws_border]),
	wccreate((stop_dialog,3),button,`Set`,90,60,50,20,
		[ws_child,ws_visible,bs_pushbutton]),
	wccreate((stop_dialog,4),button,`Cancel`,160,60,50,20,
		[ws_child,ws_visible,bs_pushbutton]),
	window_handler(stop_dialog,vstop_hndl),
	call_dialog(stop_dialog,ok).

% Instructions for Actions/Step

v_hndl(vworld,msg_menu,4002,_) :-
	assert(vstop),
	vrun.

% Instructions for Test/West

v_hndl(vworld,msg_menu,3001,_) :-
	move(w),
	veverybody_act,
	vlast(move(w)).

% Instructions for Test/East

v_hndl(vworld,msg_menu,3002,_) :-
	move(e),
	veverybody_act,
	vlast(move(e)).

% Instructions for Test/South

v_hndl(vworld,msg_menu,3003,_) :-
	move(s),
	veverybody_act,
	vlast(move(s)).

% Instructions for Test/North

v_hndl(vworld,msg_menu,3004,_) :-
	move(n),
	veverybody_act,
	vlast(move(n)).

% Instructions for Test/Sit

v_hndl(vworld,msg_menu,3000,_) :-
	sit,
	veverybody_act,
	vlast(sit).

% Instructions for Options/Change Icon

v_hndl(vworld,msg_menu,5000,_) :-
	vchange_agent_icon.

vchange_agent_icon :-
	retract(vobject_icon(a,Old)),
	vagent_icons(List),
	vnext_agent_icon(Old,List,New),
	assert(vobject_icon(a,New)),
	vagent_data(X,Y,_,_,_),
	vdraw(X,Y,a).

vnext_agent_icon(_,[],New) :-
	vagent_icons([New|_]).

vnext_agent_icon(Old,[Old,New|_],New).

vnext_agent_icon(Old,[_|Rest],New) :-
	vnext_agent_icon(Old,Rest,New).

% Instructions for Options/Change Location

v_hndl(vworld,msg_menu,5001,_) :-
	assert(vrelocate_agent).

v_hndl((vworld,1),msg_leftup,(X,Y),_) :-
	retract(vrelocate_agent),
	XX is X//32,
	YY is Y //32,
	vinspect(current,XX,YY,o),
	retract(vagent_data(XXX,YYY,S,D,I)),
	assert(vagent_data(XX,YY,S,D,I)),
	vdraw(XXX,YYY,o),
	vdraw(XX,YY,a).

% Instructions for Options/Reset Strength

v_hndl(vworld,msg_menu,5002,_) :-
	vreset_strength.

vreset_strength :-
	retract(vagent_data(X,Y,_,D,I)),
	assert(vagent_data(X,Y,100,D,I)),
	vdraw(X,Y,a),
	vstatus_report,
	!,
	vwell_and_able(_,_,_,_,_),
	% Enable Test, Actions, and Options menu items.
	wmnunbl(vtest,0,1),
	wmnunbl(vtest,1,1),
	wmnunbl(vtest,2,1),
	wmnunbl(vtest,3,1),
	wmnunbl(vtest,4,1),
	wmnunbl(vactions,0,1),
	wmnunbl(vactions,1,1),
	wmnunbl(vactions,2,1),
	wmnunbl(voptions,0,1),
	wmnunbl(voptions,1,1).

% Instructions for Options/Reset Damage

v_hndl(vworld,msg_menu,5003,_) :-
	vreset_damage.

vreset_damage :-
	retract(vagent_data(X,Y,S,D,I)),
	assert(vagent_data(X,Y,S,0,I)),
	vdraw(X,Y,a),
	vstatus_report,
	!,
	vwell_and_able(_,_,_,_,_),
	% Enable Test, Actions, and Options menu items.
	wmnunbl(vtest,0,1),
	wmnunbl(vtest,1,1),
	wmnunbl(vtest,2,1),
	wmnunbl(vtest,3,1),
	wmnunbl(vtest,4,1),
	wmnunbl(vactions,0,1),
	wmnunbl(vactions,1,1),
	wmnunbl(vactions,2,1),
	wmnunbl(voptions,0,1),
	wmnunbl(voptions,1,1).

% Instructions for Help/Contents

v_hndl(vworld,msg_menu,6000,_) :-
	help('vworld.hlp',3,0).

% Instructions for Help/About

v_hndl(vworld,msg_menu,6001,_) :-
	msgbox(`About Virtual World`,
`Virtual World~J~MCopyright 1997 Donald Nute~J~MArtificial Intelligence Center~J~MThe University of Georgia`,
	0,_).

v_hndl((vworld,1),msg_paint,_,_) :-
	vcolor_world.

% Any message not handled by the above clauses are
% passed to the standard LPA window handler.

v_hndl(Window,Message,Data,Result) :-
	window_handler(Window,Message,Data,Result).

% Window handler for the Actions/Stop dialog.

vstop_hndl((stop_dialog,3),msg_button,_,ok) :-
	wtext((stop_dialog,2),S),
	number_string(N,S),
	integer(N),
	N > 0,
	retractall(vstep(_)),
	assert(vstep(N)),
	wmnusel(vactions,1,1).

vstop_hndl((stop_dialog,3),msg_button,_,_) :-
	msgbox(`Error`,
`You should enter an integer greater than 0 as your stop value.`,
		16,_),
	wfocus((stop_dialog,2)).

vstop_hndl((stop_dialog,4),msg_button,_,ok) :-
	msgbox(`Run till you die!`,
`No maximum number of moves has been set and the action will continue until the agent dies.`,
		64,_),
	retractall(vstep(_)),
	wmnusel(vactions,1,0).

vmnuadd(Dlg,Pos,Name,MenuToAdd) :-
	wndhdl(Dlg,Handle),
	winapi((user32,'GetMenu'),[Handle],Menu),
	wmnuadd(menu(Menu),Pos,Name,MenuToAdd),
	winapi((user32,'DrawMenuBar'),[Handle],_).

vmnudel(Dlg,Pos) :-
	wndhdl(Dlg,Handle),
	winapi((user32,'GetMenu'),[Handle],Menu),
	wmnudel(menu(Menu),Pos),
	winapi((user32,'DrawMenuBar'),[Handle],_).

vmnunbl(Dlg,Pos,Status) :-
	wndhdl(Dlg,Handle),
	winapi((user32,'GetMenu'),[Handle],Menu),
	wmnunbl(menu(Menu),Pos,Status),
	winapi((user32,'DrawMenuBar'),[Handle],_).

% vcat(+List,'',-Atom) concatenates a List of atoms into Atom, placing
% an underscore between each atom in List.

vcat([],SoFar,SoFar).

vcat([A|Rest],'',Atom) :-
	!,
	vcat(Rest,A,Atom).

vcat([A|Rest],SoFar,Atom) :-
	cat([SoFar,'_',A],Next,_),
	!,
	vcat(Rest,Next,Atom).

% The abort hndl myabort/0

vabort :-
msgbox('Abort','There has been an error in this application',48,V), exit(0).
